/* dbasemy.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/

#define DBMYSQL 1
#undef DBPGSQL

#include <libiclass/dbase>
#include <unistd.h>

using namespace iclassdb;
using namespace std;

/**************************************************************************************************************/
dbconnt::dbconnt
( 
    string  Database, 
    string  User, 
    string  Password, 
    string  Host, 
    string  Port
)
{
    if ( !mysql_thread_safe() )
    {
	_failflag = true;
	_errmsg = "MySQL compiled without option --enable-thread-safe-client";

	return;
    }

    _pdbc = new dbct;
    mysql_init( _pdbc );

    unsigned  connecttimeout( 2 );

    mysql_options( _pdbc, MYSQL_OPT_CONNECT_TIMEOUT, reinterpret_cast<const char*>(&connecttimeout) );

#if MYSQL_VERSION_ID >= 50000

    my_bool   reconnect( 1 );

    mysql_options( _pdbc, MYSQL_OPT_RECONNECT, &reconnect );

#endif

    for ( int tries = 3; tries-- > 0; )
    {
    	_failflag = !mysql_real_connect
    	(
	    _pdbc,
	    Host.c_str(),
	    User.c_str(),
	    Password.c_str(),
	    Database.c_str(),
	    atoi(Port.c_str()),
	    NULL,
	    CLIENT_FOUND_ROWS | CLIENT_REMEMBER_OPTIONS
    	);

	if ( !_failflag )
	{
	    return;
	}

	usleep( 20000 );
    }

    if ( _failflag )
    {
	_errmsg = mysql_error( _pdbc );
    }
}

/**************************************************************************************************************/
dbconnt::~dbconnt()
{
    if ( !_failflag )
    {
    	mysql_close( _pdbc );
    }

    delete _pdbc;
}

/**************************************************************************************************************/
dbresultt::dbresultt( dbconnt& Db, string Query )
{
    _pdbc = Db._pdbc;
    _pdbr = NULL;
    _ri = -1;
    _failflag = Db._failflag;
    _ntuples = 0;

    if ( !_failflag )
    {
	_failflag = mysql_real_query( _pdbc, Query.data(), Query.length() );

	if ( !_failflag )
	{
	    _ntuples = mysql_affected_rows( _pdbc );

	    if ( _ntuples < 0 )
	    {
		_pdbr = mysql_store_result( _pdbc );
		_failflag = !_pdbr;
		_ntuples = _failflag? 0 : mysql_num_rows(_pdbr);
	    }
	}

    	if ( _failflag )
    	{
	    Db._errmsg = mysql_error( _pdbc );
    	}
    }
}

/**************************************************************************************************************/
dbresultt::~dbresultt()
{
    if ( _pdbr != NULL )
    {
    	mysql_free_result( _pdbr );
    }
}

/**************************************************************************************************************/
string
dbresultt::at( int Row, int Column ) const
{
    if ( _pdbr )
    {
    	if ( Row != _ri )
	{
	    if ( Row != ++_ri )
	    {
    	    	mysql_data_seek( _pdbr, Row );
		_ri = Row;
	    }
	    
    	    _row = mysql_fetch_row( _pdbr );
	}

    	if ( _row )
    	{
	    const char*	val( _row[Column] );

	    if ( val )
	    {
	    	return val;
	    }
    	}
	else
	{
	    _ri = -1;
	}
    }

    return string();
}

/**************************************************************************************************************/
bool
dbresultt::isnull( int Row, int Column ) const
{
    if ( _pdbr )
    {
    	if ( Row != _ri )
	{
	    if ( Row != ++_ri )
	    {
    	    	mysql_data_seek( _pdbr, Row );
		_ri = Row;
	    }
	    
    	    _row = mysql_fetch_row( _pdbr );
	}

    	if ( _row )
	{
	    if ( _row[Column] )
	    {
	    	return false;
	    }
    	}
	else
	{
	    _ri = -1;
	}
    }

    return true;
}
