/* dbasepg.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/

#define DBPGSQL 1
#undef DBMYSQL

#include <cstdlib>
#include <cstring>
#include <sstream>
#include <libiclass/dbase>

using namespace iclassdb;
using namespace std;

/**************************************************************************************************************/
dbconnt::dbconnt
( 
    string  Database, 
    string  User, 
    string  Password, 
    string  Host, 
    string  Port
)
{
    ostringstream  conninfooss;

    conninfooss
	<< "host='" << Host << "' "
	<< "port='" << Port << "' "
	<< "dbname='" << Database << "' "
	<< "user='" << User << "' "
	<< "password='" << Password << "' "
	<< "connect_timeout=5";

    _pdbc = PQconnectdb( conninfooss.str().c_str() );
    _failflag = !_pdbc || PQstatus(_pdbc) != CONNECTION_OK;

    if ( _pdbc && _failflag )
    {
	_errmsg = string( PQerrorMessage(_pdbc), strlen(PQerrorMessage(_pdbc)) - 1 );
    }
}

/**************************************************************************************************************/
dbconnt::~dbconnt()
{
    if ( _pdbc )
    {
    	PQfinish( _pdbc );
    }
}

/**************************************************************************************************************/
dbresultt::dbresultt( dbconnt& Db, string Query )
{
    _pdbc = Db._pdbc;
    _pdbr = NULL;
    _failflag = Db._failflag;
    _ntuples = 0;

    if ( !_failflag )
    {
	_pdbr = PQexec( Db._pdbc, Query.c_str() );
	_failflag = !_pdbr;

	if ( !_failflag )
	{
	    if ( PQresultStatus(_pdbr) == PGRES_COMMAND_OK )
	    {
	    	_ntuples = atol( PQcmdTuples(_pdbr) );
	    }
	    else if ( PQresultStatus(_pdbr) == PGRES_TUPLES_OK )
	    {
		_ntuples = PQntuples( _pdbr );
	    }
	    else
	    {
		_failflag = true;
		Db._errmsg = string( PQresultErrorMessage(_pdbr), strlen(PQresultErrorMessage(_pdbr)) - 1 );
	    }
	}
    }
}

/**************************************************************************************************************/
dbresultt::~dbresultt()
{
    if ( _pdbr != NULL )
    {
	PQclear( _pdbr );
    }
}

/**************************************************************************************************************/
string
dbresultt::at( int Row, int Column ) const
{
    return string( PQgetvalue(_pdbr,Row,Column), PQgetlength(_pdbr,Row,Column) );
}

/**************************************************************************************************************/
bool
dbresultt::isnull( int Row, int Column ) const
{
    return PQgetisnull( _pdbr, Row, Column );
}
