/* period.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/
/*
    TODO
    periodt::operator ++();
    periodt::operator --();
    periodt::operator +=( int );
    periodt::operator -=( int );

    i.e.
    	periodt  today( ltimet(time(0)), periodt::PLDAY );

	today++; // now it is tomorrow
	today-=2; // now it is yesterday
*/    

#include <libiclass/period>

using namespace iclass;
using namespace std;

/**************************************************************************************************************/
periodt::periodt( const gtimet& Begin, const gtimet& End ) :
    _begin( Begin ),
    _end( End )
{
}

/**************************************************************************************************************/
periodt::periodt( const gtimet& Point, suburbt Suburb ) :
    _begin( Point ),
    _end( Point )
{
    switch ( Suburb )
    {
        case PGYEAR:

	    _begin.setgall( Point.gyear(), 0, 0, 0, 0, 0 );
	    _end.setgall( Point.gyear()+1, 0, 0, 0, 0, 0 );

	    break;

        case PLYEAR:

	    _begin.setlall( Point.lyear(), 0, 0, 0, 0, 0 );
	    _end.setlall( Point.lyear()+1, 0, 0, 0, 0, 0 );

	    break;

        case PGYEARBEFORE:

	    _begin.setgall( Point.gyear()-1, 0, 0, 0, 0, 0 );
	    _end.setgall( Point.gyear(), 0, 0, 0, 0, 0 );

	    break;

        case PLYEARBEFORE:

	    _begin.setlall( Point.lyear()-1, 0, 0, 0, 0, 0 );
	    _end.setlall( Point.lyear(), 0, 0, 0, 0, 0 );

	    break;

        case PGYEARAFTER:

	    _begin.setgall( Point.gyear()+1, 0, 0, 0, 0, 0 );
	    _end.setgall( Point.gyear()+2, 0, 0, 0, 0, 0 );

	    break;

        case PLYEARAFTER:

	    _begin.setlall( Point.lyear()+1, 0, 0, 0, 0, 0 );
	    _end.setlall( Point.lyear()+2, 0, 0, 0, 0, 0 );

	    break;

        case PGMONTH:

	    _begin.setgall( Point.gyear(), Point.gmonth(), 0, 0, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth()+1, 0, 0, 0, 0 );

	    break;

        case PLMONTH:

	    _begin.setlall( Point.lyear(), Point.lmonth(), 0, 0, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth()+1, 0, 0, 0, 0 );

	    break;

        case PGMONTHBEFORE:

	    _begin.setgall( Point.gyear(), Point.gmonth()-1, 0, 0, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), 0, 0, 0, 0 );

	    break;

        case PLMONTHBEFORE:

	    _begin.setlall( Point.lyear(), Point.lmonth()-1, 0, 0, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), 0, 0, 0, 0 );

	    break;

        case PGMONTHAFTER:

	    _begin.setgall( Point.gyear(), Point.gmonth()+1, 0, 0, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth()+2, 0, 0, 0, 0 );

	    break;

        case PLMONTHAFTER:

	    _begin.setlall( Point.lyear(), Point.lmonth()+1, 0, 0, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth()+2, 0, 0, 0, 0 );

	    break;

	case PGDAY:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), 0, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday()+1, 0, 0, 0 );

	    break;

	case PLDAY:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), 0, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday()+1, 0, 0, 0 );

	    break;

	case PGDAYBEFORE:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday()-1, 0, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), 0, 0, 0 );

	    break;

	case PLDAYBEFORE:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday()-1, 0, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), 0, 0, 0 );

	    break;

	case PGDAYAFTER:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday()+1, 0, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday()+2, 0, 0, 0 );

	    break;

	case PLDAYAFTER:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday()+1, 0, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday()+2, 0, 0, 0 );

	    break;

	case PGHOUR:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour()+1, 0, 0 );

	    break;

	case PLHOUR:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour()+1, 0, 0 );

	    break;

	case PGHOURBEFORE:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour()-1, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), 0, 0 );

	    break;

	case PLHOURBEFORE:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour()-1, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), 0, 0 );

	    break;

	case PGHOURAFTER:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour()+1, 0, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour()+2, 0, 0 );

	    break;

	case PLHOURAFTER:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour()+1, 0, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour()+2, 0, 0 );

	    break;

	case PGMINUTE:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), Point.gminute(), 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), Point.gminute()+1, 0 );

	    break;

	case PLMINUTE:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), Point.lminute(), 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), Point.lminute()+1, 0 );

	    break;

	case PGMINUTEBEFORE:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), Point.gminute()-1, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), Point.gminute(), 0 );

	    break;

	case PLMINUTEBEFORE:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), Point.lminute()-1, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), Point.lminute(), 0 );

	    break;

	case PGMINUTEAFTER:

	    _begin.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), Point.gminute()+1, 0 );
	    _end.setgall( Point.gyear(), Point.gmonth(), Point.gmday(), Point.ghour(), Point.gminute()+2, 0 );

	    break;

	case PLMINUTEAFTER:

	    _begin.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), Point.lminute()+1, 0 );
	    _end.setlall( Point.lyear(), Point.lmonth(), Point.lmday(), Point.lhour(), Point.lminute()+2, 0 );

	    break;

	default:

	    _begin = Point;
	    _end = Point;
    }
}
