/* ipv4test.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <algorithm>
#include <iostream>
#include <limits>
#include <stdexcept>
#include <libiclass/ipv4>
#include "test.h"

using namespace std;
using namespace iclass;
using namespace iclasstest;

/**************************************************************************************************************/
bool
NumericAddressTest()
{
    testt     t( "NumericAddressTest()" );
    ipaddr4t  ip( 0 );

    //---1------------------------------------------------------------------------------------------------------
    t.check( ip.woctet()==0 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );
   
    //---2------------------------------------------------------------------------------------------------------
    ip = 16711935U; // 0.255.0.255
    t.check( ip.woctet()==0 && ip.xoctet()==255 && ip.yoctet()==0 && ip.zoctet()==255 );
    
    //---3------------------------------------------------------------------------------------------------------
    ip = 2147483647U; // 127.255.255.255
    t.check( ip.woctet()==127 && ip.xoctet()==255 && ip.yoctet()==255 && ip.zoctet()==255 );
    
    //---4------------------------------------------------------------------------------------------------------
    ip = 2147483648U; // 128.0.0.0
    t.check( ip.woctet()==128 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );
    
    //---5------------------------------------------------------------------------------------------------------
    ip = 3232238080U; // 192.168.10.0
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==10 && ip.zoctet()==0 );

    //---6------------------------------------------------------------------------------------------------------
    ip = 3232238207U; // 192.168.10.127
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==10 && ip.zoctet()==127 );

    //---7------------------------------------------------------------------------------------------------------
    ip = 3232238208U; // 192.168.10.128
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==10 && ip.zoctet()==128 );

    //---8------------------------------------------------------------------------------------------------------
    ip = 3232238335U; // 192.168.10.255
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==10 && ip.zoctet()==255 );

    //---9------------------------------------------------------------------------------------------------------
    ip = 3232238336U; // 192.168.11.0
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==11 && ip.zoctet()==0 );

    //---10-----------------------------------------------------------------------------------------------------
    ip = 4278190079U; // 254.255.255.255
    t.check( ip.woctet()==254 && ip.xoctet()==255 && ip.yoctet()==255 && ip.zoctet()==255 );
    
    //---11-----------------------------------------------------------------------------------------------------
    ip = 4278190080U; // 255.0.0.0
    t.check( ip.woctet()==255 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );
    
    //---12-----------------------------------------------------------------------------------------------------
    ip = 4294967295U; // 255.255.255.255
    t.check( ip.woctet()==255 && ip.xoctet()==255 && ip.yoctet()==255 && ip.zoctet()==255 );

    //---13-----------------------------------------------------------------------------------------------------
    ip++; // 255.255.255.255 + 1
    t.check( ip==0 && ip.woctet()==0 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );
    
    //---14-----------------------------------------------------------------------------------------------------
    ip = numeric_limits<uint32_t>::max(); // static_cast<uint32_t>(-1)
    t.check( ip.woctet()==255 && ip.xoctet()==255 && ip.yoctet()==255 && ip.zoctet()==255 );

    //---15-----------------------------------------------------------------------------------------------------
    ip++; // static_cast<ipaddr4t::uint32t>(-1) + 1
    t.check( ip==0 && ip.woctet()==0 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );
    
    //---16-----------------------------------------------------------------------------------------------------
    ip = 0;
    t.check( ip==0 && ip.woctet()==0 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );
    
    return t.result();
}

/**************************************************************************************************************/
bool
StringToAddressTest()
{
    testt     t( "StringToAddressTest()" );
    ipaddr4t  ip;

    //---1------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---2------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "ip 192.168.30.98" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---3------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "192. 168.30.98" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---4------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "192.168.30.98i" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---5------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "192.168.30.98." );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---6------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "192.168.30.98.108" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---7------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( ".168.30.98" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---8------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( " / ..." );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---9------------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "0" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---10-----------------------------------------------------------------------------------------------------
    ip = "0.0.0.0";
    t.check( ip.woctet()==0 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );
    
    //---11-----------------------------------------------------------------------------------------------------
    ip = "192.168.10.127";
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==10 && ip.zoctet()==127 );
    
    //---12-----------------------------------------------------------------------------------------------------
    ip = "192.168.10.127 ";
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==10 && ip.zoctet()==127 );
    
    //---13-----------------------------------------------------------------------------------------------------
    ip = "192.168.10.127 .108";
    t.check( ip.woctet()==192 && ip.xoctet()==168 && ip.yoctet()==10 && ip.zoctet()==127 );
    
    //---14-----------------------------------------------------------------------------------------------------
    ip = "10.0.0.1";
    
    try
    {
    	ip = "192.168.10.128/25";
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( ip.woctet()==10 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==1 );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---15-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( " 192.168.11" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---16-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( " 192.168.11\t" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
    //---17-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "\t\n\v\f\r192.169\t" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---18-----------------------------------------------------------------------------------------------------
    ip = "\t\n\v\f\r192.169.0.0\t";
    t.check( ip.woctet()==192 && ip.xoctet()==169 && ip.yoctet()==0 && ip.zoctet()==0 );
    
    //---19-----------------------------------------------------------------------------------------------------
    ip = "10.0.0.1";
    
    try
    {
    	ip = "192.169.";
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( ip.woctet()==10 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==1 );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---20-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "192.169" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---21-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "  192 . " );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---22-----------------------------------------------------------------------------------------------------
    ip = "  192.0.0.0 . ";
    t.check( ip.woctet()==192 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );

    //---23-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "0010" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---24-----------------------------------------------------------------------------------------------------
    ip = "0010.0000.0000.0000";
    t.check( ip.woctet()==10 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );

    //---25-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "10 11" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---26-----------------------------------------------------------------------------------------------------
    ip = "10.0.0.0 11";
    t.check( ip.woctet()==10 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );

    //---27-----------------------------------------------------------------------------------------------------
    try
    {
    	ipaddr4t( "15/8" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---28-----------------------------------------------------------------------------------------------------
    ip = "15.0.0.0 /8";
    t.check( ip.woctet()==15 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==0 );

    //---29-----------------------------------------------------------------------------------------------------
    ip = "10.0.0.1";
    
    try
    {
    	ip = "192.256.0.10";
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( ip.woctet()==10 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==1 );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---30-----------------------------------------------------------------------------------------------------
    try
    {
    	ip = "192.200.0.-1";
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( ip.woctet()==10 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==1 );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---31-----------------------------------------------------------------------------------------------------
    try
    {
    	ip = "-120.200.0.1";
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check( ip.woctet()==10 && ip.xoctet()==0 && ip.yoctet()==0 && ip.zoctet()==1 );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---32-----------------------------------------------------------------------------------------------------
    ip = "10.182.15.11";
    t.check( ip == ipaddr4t("010.182.015.011") );

    //---33-----------------------------------------------------------------------------------------------------
    ip = "0.0.0.255";
    t.check( ip == 255 );

    return t.result();
}

/**************************************************************************************************************/
bool
AddressToStringTest()
{
    testt  t( "AddressToStringTest()" );

    //---1------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("10.0.0.0 /8").str() == "10.0.0.0" );

    //---2------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("192.168.010.000").str() == "192.168.10.0" );

    //---3------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t().str() == "0.0.0.0" );

    return t.result();
}

/**************************************************************************************************************/
bool
AddressComparisonTest()
{
    testt  t( "AddressComparisonTest()" );

    //---1------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("10.0.0.0") == ipaddr4t("10.0.0.0") );

    //---2------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("0.0.0.0") == ipaddr4t("0.0.0.0") );

    //---3------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("255.255.255.255") == ipaddr4t("255.255.255.255") );

    //---4------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("10.0.0.0") != ipaddr4t("10.0.0.1") );

    //---5------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("0.0.0.0") != ipaddr4t("10.0.0.1") );

    //---6------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("10.0.0.0") != ipaddr4t("255.255.255.255") );

    //---7------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("10.0.0.0") < ipaddr4t("10.0.0.1") );

    //---8------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("0.0.0.0") < ipaddr4t("10.0.0.1") );

    //---9------------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("10.0.0.0") <= ipaddr4t("10.0.0.1") );

    //---10-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("10.0.0.0") <= ipaddr4t("10.0.0.0") );

    //---11-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("0.0.0.0") <= ipaddr4t("10.0.0.0") );

    //---12-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("0.0.0.0") <= ipaddr4t("255.255.255.255") );

    //---13-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("192.168.10.196") > ipaddr4t("0.0.0.0") );

    //---14-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("192.168.10.196") > ipaddr4t("192.168.10.195") );

    //---15-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("192.168.10.196") >= ipaddr4t("192.168.10.195") );

    //---16-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("192.168.10.196") >= ipaddr4t("192.168.10.196") );

    //---17-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("255.255.255.255") >= ipaddr4t("192.168.10.196") );

    //---18-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("255.255.255.255") >= ipaddr4t("255.255.255.255") );

    set<ipaddr4t>  s;

    s.size();	// this is test to equal_to<ipaddr4t> and less<ipaddr4t> functionals availability

    return t.result();
}

/**************************************************************************************************************/
bool
AddressOperatorsTest()
{
    testt     t( "AddressOperatorsTest()" );
    ipaddr4t  ip( "192.168.10.196" );

    //---1------------------------------------------------------------------------------------------------------
    t.check( ip++ == ipaddr4t("192.168.10.196") );

    //---2------------------------------------------------------------------------------------------------------
    t.check( ip == ipaddr4t("192.168.10.197") );

    //---3------------------------------------------------------------------------------------------------------
    t.check( ++ip == ipaddr4t("192.168.10.198") );

    //---4------------------------------------------------------------------------------------------------------
    t.check( ip == ipaddr4t("192.168.10.198") );

    //---5------------------------------------------------------------------------------------------------------
    t.check( ip-- == ipaddr4t("192.168.10.198") );

    //---6------------------------------------------------------------------------------------------------------
    t.check( ip == ipaddr4t("192.168.10.197") );

    //---7------------------------------------------------------------------------------------------------------
    t.check( --ip == ipaddr4t("192.168.10.196") );

    //---8------------------------------------------------------------------------------------------------------
    t.check( ip == ipaddr4t("192.168.10.196") );

    //---9------------------------------------------------------------------------------------------------------
    ip = "255.255.255.255";
    t.check( ++ip == ipaddr4t("0.0.0.0") );

    //---10-----------------------------------------------------------------------------------------------------
    t.check( --ip == ipaddr4t("255.255.255.255") );

    //---11-----------------------------------------------------------------------------------------------------
    ip += 2;
    t.check( ip == ipaddr4t("0.0.0.1") );

    //---12-----------------------------------------------------------------------------------------------------
    ip -= 4;
    t.check( ip == ipaddr4t("255.255.255.253") );

    //---13-----------------------------------------------------------------------------------------------------
    t.check( ip + 2 == ipaddr4t("255.255.255.255") );

    //---14-----------------------------------------------------------------------------------------------------
    t.check( ip - 3 == ipaddr4t("255.255.255.250") );

    //---15-----------------------------------------------------------------------------------------------------
    t.check( ip - ipaddr4t("255.255.255.250") == 3 );

    //---16-----------------------------------------------------------------------------------------------------
    t.check( ipaddr4t("255.255.255.250") - ip == -3 );

    return t.result();
}

/**************************************************************************************************************/
bool
NumericNetworkTest()
{
    testt    t( "NumericNetworkTest()" );
    ipnet4t  ipn;

    //---1------------------------------------------------------------------------------------------------------
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "0.0.0.0" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "255.255.255.255" && 
	ipn.networkbits() == 0 &&
	ipn.nodebits() == 32
    );

    //---2------------------------------------------------------------------------------------------------------
    ipn = ipaddr4t( "0.0.0.0" );
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "0.0.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    
    //---3------------------------------------------------------------------------------------------------------
    ipn = ipnet4t( ipaddr4t("0.0.0.0"), ipaddr4t("0.0.0.0") );
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "0.0.0.0" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "255.255.255.255" && 
	ipn.networkbits() == 0 &&
	ipn.nodebits() == 32
    );

    //---4------------------------------------------------------------------------------------------------------
    ipn = ipnet4t( ipaddr4t("192.168.51.128"), ipaddr4t("255.255.255.128") );
    t.check
    (
    	ipn.address().str() == "192.168.51.128" &&
	ipn.mask().str() == "255.255.255.128" &&
    	ipn.front().str() == "192.168.51.128" && 
	ipn.back().str() == "192.168.51.255" && 
	ipn.networkbits() == 25 &&
	ipn.nodebits() == 7
    );
    
    //---5------------------------------------------------------------------------------------------------------
    ipn = ipnet4t( ipaddr4t("192.168.51.128"), ipaddr4t("255.255.255.128") );

    try
    {
    	ipn = ipnet4t( ipaddr4t("217.18.60.128"), ipaddr4t("255.255.255.0") );
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check
    	(
    	    ipn.address().str() == "192.168.51.128" &&
	    ipn.mask().str() == "255.255.255.128" &&
    	    ipn.front().str() == "192.168.51.128" && 
	    ipn.back().str() == "192.168.51.255" && 
	    ipn.networkbits() == 25 &&
	    ipn.nodebits() == 7
    	);
    }
    catch ( ... )
    {
	t.check( false );
    }

//cout << ipn.address() << " " << ipn.mask() << " " << ipn.front() << " " << ipn.back() << " " << ipn.networkbits() << " " << ipn.nodebits() << endl;

    return t.result();
}

/**************************************************************************************************************/
bool
StringToNetworkTest()
{
    testt    t( "StringToNetworkTest()" );
    ipnet4t  ipn;
    
    //---1------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---2------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "ip 192.168.30.98" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---3------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "192. 168.30.98" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---4------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "192.168.30.98i" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---5------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "192.168.30.98." );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---6------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "192.168.30.98.108" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    //---7------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( ".168.30.98" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---8------------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( " / ..." );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---9------------------------------------------------------------------------------------------------------
    ipn = "0.0.0.0";
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "0.0.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );
    
    //---10-----------------------------------------------------------------------------------------------------
    ipn = "192.168.10.127";
    t.check
    (
    	ipn.address().str() == "192.168.10.127" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.168.10.127" && 
	ipn.back().str() == "192.168.10.127" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );
    
    //---11-----------------------------------------------------------------------------------------------------
    ipn = "192.168.10.127 ";
    t.check
    (
    	ipn.address().str() == "192.168.10.127" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.168.10.127" && 
	ipn.back().str() == "192.168.10.127" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );
    
    //---12-----------------------------------------------------------------------------------------------------
    ipn = "192.168.10.127 .108";
    t.check
    (
    	ipn.address().str() == "192.168.10.127" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.168.10.127" && 
	ipn.back().str() == "192.168.10.127" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );
    
    //---13-----------------------------------------------------------------------------------------------------
    ipn = "192.168.10.128/25";
    t.check
    (
    	ipn.address().str() == "192.168.10.128" &&
	ipn.mask().str() == "255.255.255.128" &&
    	ipn.front().str() == "192.168.10.128" && 
	ipn.back().str() == "192.168.10.255" && 
	ipn.networkbits() == 25 &&
	ipn.nodebits() == 7
    );
    
    //---14-----------------------------------------------------------------------------------------------------
    ipn = string( " 192.168.11" );
    t.check
    (
    	ipn.address().str() == "192.168.11.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.168.11.0" && 
	ipn.back().str() == "192.168.11.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );
    
    //---15-----------------------------------------------------------------------------------------------------
    ipn = "\t\n\v\f\r192.169\t";
    t.check
    (
    	ipn.address().str() == "192.169.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.169.0.0" && 
	ipn.back().str() == "192.169.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );
    
    //---16-----------------------------------------------------------------------------------------------------
    ipn = "10.0.0.1";
    
    try
    {
    	ipn = "192.169.";
	t.check( false );
    }
    catch ( invalid_argument )
    {
    	t.check
    	(
    	    ipn.address().str() == "10.0.0.1" &&
	    ipn.mask().str() == "255.255.255.255" &&
    	    ipn.front().str() == "10.0.0.1" && 
	    ipn.back().str() == "10.0.0.1" && 
	    ipn.networkbits() == 32 &&
	    ipn.nodebits() == 0
    	);
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---17-----------------------------------------------------------------------------------------------------
    ipn = "  192 . ";
    t.check
    (
    	ipn.address().str() == "192.0.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.0.0.0" && 
	ipn.back().str() == "192.0.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );
    
    //---18-----------------------------------------------------------------------------------------------------
    ipn = "0010";
    t.check
    (
    	ipn.address().str() == "10.0.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "10.0.0.0" && 
	ipn.back().str() == "10.0.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---19-----------------------------------------------------------------------------------------------------
    ipn = "10 11";
    t.check
    (
    	ipn.address().str() == "10.0.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "10.0.0.0" && 
	ipn.back().str() == "10.0.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---20-----------------------------------------------------------------------------------------------------
    ipn = "15/8";
    t.check
    (
    	ipn.address().str() == "15.0.0.0" &&
	ipn.mask().str() == "255.0.0.0" &&
    	ipn.front().str() == "15.0.0.0" && 
	ipn.back().str() == "15.255.255.255" && 
	ipn.networkbits() == 8 &&
	ipn.nodebits() == 24
    );

    //---21-----------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "192.256.0.10" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---22-----------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "192.200.0.-1" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---23-----------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "-120.200.0.1" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
    
    //---24-----------------------------------------------------------------------------------------------------
    ipn = "010.182.015.011";
    t.check
    (
    	ipn.address().str() == "10.182.15.11" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "10.182.15.11" && 
	ipn.back().str() == "10.182.15.11" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---25-----------------------------------------------------------------------------------------------------
    ipn = "0.0.0.255";
    t.check
    (
    	ipn.address().str() == "0.0.0.255" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "0.0.0.255" && 
	ipn.back().str() == "0.0.0.255" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---26-----------------------------------------------------------------------------------------------------
    ipn = "0";
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "0.0.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---27-----------------------------------------------------------------------------------------------------
    ipn = "0.0.0.0/0";
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "0.0.0.0" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "255.255.255.255" && 
	ipn.networkbits() == 0 &&
	ipn.nodebits() == 32
    );

    //---28-----------------------------------------------------------------------------------------------------
    ipn = "0/0";
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "0.0.0.0" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "255.255.255.255" && 
	ipn.networkbits() == 0 &&
	ipn.nodebits() == 32
    );

    //---29-----------------------------------------------------------------------------------------------------
    ipn = "0/1";
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "128.0.0.0" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "127.255.255.255" && 
	ipn.networkbits() == 1 &&
	ipn.nodebits() == 31
    );

    //---30-----------------------------------------------------------------------------------------------------
    ipn = "0/32";
    t.check
    (
    	ipn.address().str() == "0.0.0.0" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "0.0.0.0" && 
	ipn.back().str() == "0.0.0.0" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---31-----------------------------------------------------------------------------------------------------
    ipn = "192.168.33.8/31";
    t.check
    (
    	ipn.address().str() == "192.168.33.8" &&
	ipn.mask().str() == "255.255.255.254" &&
    	ipn.front().str() == "192.168.33.8" && 
	ipn.back().str() == "192.168.33.9" && 
	ipn.networkbits() == 31 &&
	ipn.nodebits() == 1
    );

    //---32-----------------------------------------------------------------------------------------------------
    ipn = "192.168.33.8/32";
    t.check
    (
    	ipn.address().str() == "192.168.33.8" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.168.33.8" && 
	ipn.back().str() == "192.168.33.8" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---33-----------------------------------------------------------------------------------------------------
    ipn = "192.168.33.7/32";
    t.check
    (
    	ipn.address().str() == "192.168.33.7" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "192.168.33.7" && 
	ipn.back().str() == "192.168.33.7" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---34-----------------------------------------------------------------------------------------------------
    ipn = "255.255.255.254/31";
    t.check
    (
    	ipn.address().str() == "255.255.255.254" &&
	ipn.mask().str() == "255.255.255.254" &&
    	ipn.front().str() == "255.255.255.254" && 
	ipn.back().str() == "255.255.255.255" && 
	ipn.networkbits() == 31 &&
	ipn.nodebits() == 1
    );

    //---35-----------------------------------------------------------------------------------------------------
    ipn = "255.255.255.255";
    t.check
    (
    	ipn.address().str() == "255.255.255.255" &&
	ipn.mask().str() == "255.255.255.255" &&
    	ipn.front().str() == "255.255.255.255" && 
	ipn.back().str() == "255.255.255.255" && 
	ipn.networkbits() == 32 &&
	ipn.nodebits() == 0
    );

    //---36-----------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "255.255.255.255/31" );
	t.check( false );
    }
    catch ( invalid_argument )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    return t.result();
}

/**************************************************************************************************************/
bool
NetworkToStringTest()
{
    testt    t( "NetworkToStringTest()" );
    ipnet4t  ipn;
    
    //---1------------------------------------------------------------------------------------------------------
    ipn = "192.168.51.88/29";
    t.check( ipn.str() == "192.168.51.88/29" );
    
    //---2------------------------------------------------------------------------------------------------------
    ipn = "192.168.51.0/29";
    t.check( ipn.str() == "192.168.51.0/29" );
    
    //---3------------------------------------------------------------------------------------------------------
    ipn = "192.168.51.0/24";
    t.check( ipn.str() == "192.168.51/24" );
    
    //---4------------------------------------------------------------------------------------------------------
    ipn = "192.168.51.128/25";
    t.check( ipn.str() == "192.168.51.128/25" );
    
    //---5------------------------------------------------------------------------------------------------------
    ipn = "0.0.0.0/25";
    t.check( ipn.str() == "0.0.0.0/25" );
    
    //---6------------------------------------------------------------------------------------------------------
    ipn = "0.0.0.0/24";
    t.check( ipn.str() == "0.0.0/24" );
    
    //---7------------------------------------------------------------------------------------------------------
    ipn = "0.0.0.0/0";
    t.check( ipn.str() == "0/0" );
    
    return t.result();
}

/**************************************************************************************************************/
bool
NetworkComparisonTest()
{
    testt  t( "NetworkComparisonTest()" );

    //---1------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0") == ipnet4t("10.0.0.0") );

    //---2------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/8") == ipnet4t("10.0.0.0/8") );

    //---3------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0") == ipnet4t("0.0.0.0") );

    //---4------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0") == ipnet4t("0.0.0.0/0") );

    //---5------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0") == ipnet4t("0.0.0.0/32") );

    //---6------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255") == ipnet4t("255.255.255.255") );

    //---7------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255") == ipnet4t("255.255.255.255/32") );

    //---8------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0") != ipnet4t("10.0.0.1") );

    //---9------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/8") != ipnet4t("10.0.0.0") );

    //---10-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0") != ipnet4t("10.0.0.1") );

    //---11-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0") != ipnet4t("0.0.0.0/1") );

    //---12-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0") != ipnet4t("255.255.255.255") );

    //---13-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30") != ipnet4t("255.255.255.252") );

    //---14-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0") < ipnet4t("10.0.0.1") );

    //---15-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/30") < ipnet4t("10.0.0.0") );

    //---16-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/8") < ipnet4t("10.0.0.4/30") );

    //---17-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/29") < ipnet4t("10.0.0.0/30") );

    //---18-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0") < ipnet4t("10.0.0.1") );

    //---19-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0") < ipnet4t("0.0.0.0") );

    //---20-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0") <= ipnet4t("10.0.0.1") );

    //---21-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/8") <= ipnet4t("10.0.0.1") );

    //---22-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/8") <= ipnet4t("10.0.0.4/30") );

    //---23-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0") <= ipnet4t("10.0.0.0") );

    //---24-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0/30") <= ipnet4t("10.0.0.0/30") );

    //---25-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0") <= ipnet4t("10.0.0.0") );

    //---26-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/1") <= ipnet4t("10.0.0.0/8") );

    //---27-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/8") <= ipnet4t("10.0.0.0/8") );

    //---28-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/32") <= ipnet4t("10.0.0.0/8") );

    //---29-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0") <= ipnet4t("255.255.255.255") );

    //---30-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/32") <= ipnet4t("255.255.255.255/32") );

    //---31-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0") <= ipnet4t("255.255.255.252/30") );

    //---32-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30") <= ipnet4t("255.255.255.252/30") );

    //---33-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255/32") <= ipnet4t("255.255.255.255/32") );

    //---34-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.254/31") <= ipnet4t("255.255.255.254/31") );

    //---35-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.254/31") <= ipnet4t("255.255.255.254/32") );

    //---36-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196") > ipnet4t("0.0.0.0") );

    //---37-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196") > ipnet4t("192.168.10.196/30") );

    //---38-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30") > ipnet4t("192.168.10.196/30") );

    //---39-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252") > ipnet4t("255.255.255.252/30") );

    //---40-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196") > ipnet4t("192.168.10.195") );

    //---41-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196/30") > ipnet4t("192.168.10.192/30") );

    //---42-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.192/30") > ipnet4t("192.168.10.192/29") );

    //---43-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196") >= ipnet4t("192.168.10.195") );

    //---44-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196/30") >= ipnet4t("192.168.10.195") );

    //---45-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196/30") >= ipnet4t("192.168.10.196/30") );

    //---46-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.196") >= ipnet4t("192.168.10.196") );

    //---47-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255") >= ipnet4t("192.168.10.196") );

    //---48-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255") >= ipnet4t("192.168.10.196/30") );

    //---49-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255") >= ipnet4t("0.0.0.0/0") );

    //---50-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255") >= ipnet4t("0.0.0.0/32") );

    //---51-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255") >= ipnet4t("255.255.255.255") );

    //---52-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30") >= ipnet4t("255.255.255.252/30") );

    set<ipnet4t>  s;

    s.size();	// this is test to equal_to<ipnet4t> and less<ipnet4t> functionals availability

    return t.result();
}

/**************************************************************************************************************/
bool
NetworkInclusionTest()
{
    testt  t( "NetworkInclusionTest()" );

    //---1------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0").contains(ipaddr4t("10.0.0.0")) );

    //---2------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").contains(ipaddr4t("10.0.0.1")) );

    //---3------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").contains(ipaddr4t("9.255.255.255")) );

    //---4------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0").contains(ipaddr4t("0.0.0.0")) );

    //---5------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").contains(ipaddr4t("0.0.0.1")) );

    //---6------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").contains(ipaddr4t("255.255.255.255")) );

    //---7------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255").contains(ipaddr4t("255.255.255.255")) );

    //---8------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").contains(ipaddr4t("0.0.0.0")) );

    //---9------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").contains(ipaddr4t("255.255.255.254")) );

    //---10-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/26").contains(ipaddr4t("192.168.10.63")) );

    //---11-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/26").contains(ipaddr4t("192.168.10.64")) );

    //---12-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/26").contains(ipaddr4t("192.168.10.100")) );

    //---13-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/26").contains(ipaddr4t("192.168.10.127")) );

    //---14-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/26").contains(ipaddr4t("192.168.10.128")) );

    //---15-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").contains(ipaddr4t("255.255.255.255")) );

    //---16-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipaddr4t("0.0.0.0")) );

    //---17-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipaddr4t("0.0.0.1")) );

    //---18-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipaddr4t("0.0.0.2")) );

    //---19-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipaddr4t("0.0.0.3")) );

    //---20-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").contains(ipaddr4t("0.0.0.4")) );

    //---21-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").contains(ipaddr4t("255.255.255.251")) );

    //---22-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipaddr4t("255.255.255.252")) );

    //---23-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipaddr4t("255.255.255.253")) );

    //---24-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipaddr4t("255.255.255.254")) );

    //---25-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipaddr4t("255.255.255.255")) );

    //---26-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").contains(ipaddr4t("0.0.0.0")) );

    //---27-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipaddr4t("0.0.0.0")) );

    //---28-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipaddr4t("0.0.0.1")) );

    //---29-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipaddr4t("255.255.255.254")) );

    //---30-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipaddr4t("255.255.255.255")) );

    //---31-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0").contains(ipnet4t("10.0.0.0/32")) );

    //---32-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").contains(ipnet4t("10.0.0.0/31")) );

    //---33-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").contains(ipnet4t("10.0.0.1")) );

    //---34-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").contains(ipnet4t("9.255.255.255")) );

    //---35-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255").contains(ipnet4t("255.255.255.255/32")) );

    //---36-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").contains(ipnet4t("255.255.255.254")) );

    //---37-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").contains(ipnet4t("0.0.0.0")) );

    //---38-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.60/30")) );

    //---39-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.63")) );

    //---40-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.64")) );

    //---41-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.64/30")) );

    //---42-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.64/27")) );

    //---43-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.64/26")) );

    //---44-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.80/30")) );

    //---45-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.88/29")) );

    //---46-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.95")) );

    //---47-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.96")) );

    //---48-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").contains(ipnet4t("192.168.10.96/30")) );

    //---49-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").contains(ipnet4t("255.255.255.252/30")) );

    //---50-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").contains(ipnet4t("255.255.255.255")) );

    //---51-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.0")) );

    //---52-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.0/31")) );

    //---53-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.0/30")) );

    //---54-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.0/29")) );

    //---55-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.1")) );

    //---56-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.2")) );

    //---57-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.2/31")) );

    //---58-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.3")) );

    //---59-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.4")) );

    //---60-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").contains(ipnet4t("0.0.0.4/30")) );

    //---61-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.248/30")) );

    //---62-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.251")) );

    //---63-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.252")) );

    //---64-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.252/31")) );

    //---65-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.252/30")) );

    //---66-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.253")) );

    //---67-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.254")) );

    //---68-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.254/31")) );

    //---69-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").contains(ipnet4t("255.255.255.255")) );

    //---70-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").contains(ipnet4t("0.0.0.0")) );

    //---71-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").contains(ipnet4t("0.0.0.0/30")) );

    //---72-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("0.0.0.0")) );

    //---73-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("0.0.0.0/8")) );

    //---74-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("0.0.0.0/0")) );

    //---75-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("0.0.0.1")) );

    //---76-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("10.0.0.0/8")) );

    //---77-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("255.255.255.252/30")) );

    //---78-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("255.255.255.254")) );

    //---79-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/0").contains(ipnet4t("255.255.255.255")) );

    return t.result();
}

/**************************************************************************************************************/
bool
NetworkAdjacentTest()
{
    testt  t( "NetworkAdjacentTest()" );

    //---1------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0").isadjacent(ipnet4t("9.255.255.252/30")) );

    //---2------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").isadjacent(ipnet4t("9.255.255.254")) );

    //---3------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0").isadjacent(ipnet4t("9.255.255.255")) );

    //---4------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").isadjacent(ipnet4t("10.0.0.0")) );

    //---5------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0").isadjacent(ipnet4t("10.0.0.1")) );

    //---6------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").isadjacent(ipnet4t("10.0.0.2")) );

    //---7------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").isadjacent(ipnet4t("10.0.0.2/31")) );

    //---8------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").isadjacent(ipnet4t("255.255.255.252/30")) );

    //---9------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").isadjacent(ipnet4t("255.255.255.255")) );

    //---10-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").isadjacent(ipnet4t("0.0.0.0")) );

    //---11-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0").isadjacent(ipnet4t("0.0.0.1")) );

    //---12-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").isadjacent(ipnet4t("0.0.0.2")) );

    //---13-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").isadjacent(ipnet4t("0.0.0.2/31")) );

    //---14-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").isadjacent(ipnet4t("255.255.255.252/30")) );

    //---15-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").isadjacent(ipnet4t("255.255.255.253")) );

    //---16-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255").isadjacent(ipnet4t("255.255.255.254")) );

    //---17-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").isadjacent(ipnet4t("255.255.255.255")) );

    //---18-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").isadjacent(ipnet4t("0.0.0.0")) );

    //---19-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").isadjacent(ipnet4t("0.0.0.0/30")) );

    //---20-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.60/30")) );

    //---21-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.60/31")) );

    //---22-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.62")) );

    //---23-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.63")) );

    //---24-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.64")) );

    //---25-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.64/27")) );

    //---26-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.64/26")) );

    //---27-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.80/29")) );

    //---28-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.80/28")) );

    //---29-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.94")) );

    //---30-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.95")) );

    //---31-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.96")) );

    //---32-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.96/27")) );

    //---33-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").isadjacent(ipnet4t("192.168.10.100/30")) );

    //---34-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("255.255.255.252/30")) );

    //---35-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("255.255.255.254")) );

    //---36-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("255.255.255.254/31")) );

    //---37-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("255.255.255.255")) );

    //---38-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.0")) );

    //---39-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.0/30")) );

    //---40-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.0/0")) );

    //---41-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.2/31")) );

    //---42-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.3")) );

    //---43-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.4")) );

    //---44-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.4/30")) );

    //---45-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.5")) );

    //---46-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").isadjacent(ipnet4t("0.0.0.6/31")) );

    //---47-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.248/30")) );

    //---48-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.250")) );

    //---50-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.250/31")) );

    //---51-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.251")) );

    //---52-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.252")) );

    //---53-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.252/30")) );

    //---54-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.254")) );

    //---55-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("255.255.255.255")) );

    //---56-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("0.0.0.0")) );

    //---57-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("0.0.0.0/8")) );

    //---58-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("0.0.0.0/0")) );

    //---59-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").isadjacent(ipnet4t("0.0.0.4/30")) );

    //---60-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("0.0.0.0")) );

    //---61-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("0.0.0.0/8")) );

    //---62-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("0.0.0.0/0")) );

    //---63-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("0.0.0.1")) );

    //---64-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("10.0.0.0/8")) );

    //---65-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("255.255.255.252/30")) );

    //---66-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("255.255.255.254")) );

    //---67-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").isadjacent(ipnet4t("255.255.255.255")) );

    return t.result();
}

/**************************************************************************************************************/
bool
NetworkMergeableTest()
{
    testt  t( "NetworkMergeableTest()" );

    //---1------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").ismergeable(ipnet4t("9.255.255.252/30")) );

    //---2------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").ismergeable(ipnet4t("9.255.255.254")) );

    //---3------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").ismergeable(ipnet4t("9.255.255.255")) );

    //---4------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").ismergeable(ipnet4t("10.0.0.0")) );

    //---5------------------------------------------------------------------------------------------------------
    t.check( ipnet4t("10.0.0.0").ismergeable(ipnet4t("10.0.0.1")) );

    //---6------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").ismergeable(ipnet4t("10.0.0.2")) );

    //---7------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("10.0.0.0").ismergeable(ipnet4t("10.0.0.2/31")) );

    //---8------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").ismergeable(ipnet4t("255.255.255.252/30")) );

    //---9------------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").ismergeable(ipnet4t("255.255.255.255")) );

    //---10-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").ismergeable(ipnet4t("0.0.0.0")) );

    //---11-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0").ismergeable(ipnet4t("0.0.0.1")) );

    //---12-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").ismergeable(ipnet4t("0.0.0.2")) );

    //---13-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0").ismergeable(ipnet4t("0.0.0.2/31")) );

    //---14-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").ismergeable(ipnet4t("255.255.255.252/30")) );

    //---15-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").ismergeable(ipnet4t("255.255.255.253")) );

    //---16-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.255").ismergeable(ipnet4t("255.255.255.254")) );

    //---17-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").ismergeable(ipnet4t("255.255.255.255")) );

    //---18-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").ismergeable(ipnet4t("0.0.0.0")) );

    //---19-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.255").ismergeable(ipnet4t("0.0.0.0/30")) );

    //---20-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.0/26")) );

    //---21-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.32/27")) );

    //---22-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.62/31")) );

    //---23-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.63")) );

    //---24-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.64")) );

    //---25-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.64/27")) );

    //---26-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.64/26")) );

    //---27-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.80/29")) );

    //---28-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.80/28")) );

    //---29-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.94")) );

    //---30-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.95")) );

    //---31-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.96")) );

    //---32-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.96/28")) );

    //---33-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.96/27")) );

    //---34-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("192.168.10.64/27").ismergeable(ipnet4t("192.168.10.100/30")) );

    //---35-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("255.255.255.252/30")) );

    //---36-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("255.255.255.254")) );

    //---37-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("255.255.255.254/31")) );

    //---38-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("255.255.255.255")) );

    //---39-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.0")) );

    //---40-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.0/30")) );

    //---41-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.0/0")) );

    //---42-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.2/31")) );

    //---43-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.3")) );

    //---44-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.4")) );

    //---45-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.4/30")) );

    //---46-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.5")) );

    //---47-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/30").ismergeable(ipnet4t("0.0.0.6/31")) );

    //---48-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.240/28")) );

    //---49-----------------------------------------------------------------------------------------------------
    t.check( ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.248/30")) );

    //---50-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.250")) );

    //---51-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.250/31")) );

    //---52-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.251")) );

    //---53-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.252")) );

    //---54-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.252/30")) );

    //---55-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.254")) );

    //---56-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("255.255.255.255")) );

    //---57-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("0.0.0.0")) );

    //---58-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("0.0.0.0/8")) );

    //---59-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("0.0.0.0/0")) );

    //---60-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("255.255.255.252/30").ismergeable(ipnet4t("0.0.0.4/30")) );

    //---61-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("0.0.0.0")) );

    //---62-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("0.0.0.0/8")) );

    //---63-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("0.0.0.0/0")) );

    //---64-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("0.0.0.1")) );

    //---65-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("10.0.0.0/8")) );

    //---66-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("255.255.255.252/30")) );

    //---67-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("255.255.255.254")) );

    //---68-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("0.0.0.0/0").ismergeable(ipnet4t("255.255.255.255")) );

    //---69-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("81.17.162.96/28").ismergeable(ipnet4t("81.17.162.80/28")) );

    //---70-----------------------------------------------------------------------------------------------------
    t.check( !ipnet4t("81.17.162.80/28").ismergeable(ipnet4t("81.17.162.96/28")) );

    return t.result();
}

/**************************************************************************************************************/
bool
NetworkMergeTest()
{
    testt  t( "NetworkMergeTest()" );

    //---1------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("10.0.0.0").merge() == ipnet4t("10.0.0.0/31") &&
	ipnet4t("10.0.0.0").merge().back() == ipaddr4t("10.0.0.1")
    );

    //---2------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("10.0.0.1").merge() == ipnet4t("10.0.0.0/31") &&
	ipnet4t("10.0.0.1").merge().back() == ipaddr4t("10.0.0.1")
    );

    //---3------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("0.0.0.0").merge() == ipnet4t("0.0.0.0/31") &&
	ipnet4t("0.0.0.0").merge().back() == ipaddr4t("0.0.0.1")
    );

    //---4------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("0.0.0.1").merge() == ipnet4t("0.0.0.0/31") &&
	ipnet4t("0.0.0.1").merge().back() == ipaddr4t("0.0.0.1")
    );

    //---5------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("255.255.255.254").merge() == ipnet4t("255.255.255.254/31") &&
	ipnet4t("255.255.255.254").merge().back() == ipaddr4t("255.255.255.255")
    );

    //---6------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("255.255.255.255").merge() == ipnet4t("255.255.255.254/31") &&
	ipnet4t("255.255.255.255").merge().back() == ipaddr4t("255.255.255.255")
    );

    //---7------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("192.168.10.64/27").merge() == ipnet4t("192.168.10.64/26") &&
	ipnet4t("192.168.10.64/27").merge().back() == ipaddr4t("192.168.10.127")
    );

    //---8------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("192.168.10.96/27").merge() == ipnet4t("192.168.10.64/26") &&
	ipnet4t("192.168.10.96/27").merge().back() == ipaddr4t("192.168.10.127")
    );

    //---9------------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("0.0.0.0/30").merge() == ipnet4t("0.0.0.0/29") &&
	ipnet4t("0.0.0.0/30").merge().back() == ipaddr4t("0.0.0.7")
    );

    //---10-----------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("0.0.0.4/30").merge() == ipnet4t("0.0.0.0/29") &&
	ipnet4t("0.0.0.4/30").merge().back() == ipaddr4t("0.0.0.7")
    );

    //---11-----------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("255.255.255.248/30").merge() == ipnet4t("255.255.255.248/29") &&
	ipnet4t("255.255.255.248/30").merge().back() == ipaddr4t("255.255.255.255")
    );

    //---12-----------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("255.255.255.252/30").merge() == ipnet4t("255.255.255.248/29") &&
	ipnet4t("255.255.255.252/30").merge().back() == ipaddr4t("255.255.255.255")
    );

    //---13-----------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("81.17.162.96/28").merge() == ipnet4t("81.17.162.96/27") &&
	ipnet4t("81.17.162.96/28").merge().back() == ipaddr4t("81.17.162.127")
    );

    //---14-----------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("0.0.0.0/1").merge() == ipnet4t("0.0.0.0/0") &&
	ipnet4t("0.0.0.0/1").merge().back() == ipaddr4t("255.255.255.255")
    );

    //---15-----------------------------------------------------------------------------------------------------
    t.check
    (
	ipnet4t("128.0.0.0/1").merge() == ipnet4t("0.0.0.0/0") &&
	ipnet4t("128.0.0.0/1").merge().back() == ipaddr4t("255.255.255.255")
    );

    //---16-----------------------------------------------------------------------------------------------------
    try
    {
    	ipnet4t( "0.0.0.0/0" ).merge();
	t.check( false );
    }
    catch ( overflow_error )
    {
	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }

    return t.result();
}

/**************************************************************************************************************/
bool
SetInsertTest()
{
    testt  t( "SetInsertTest()" );

    //---1------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("10.0.0.0") );
	s.insert( ipnet4t("0.0.0.0") );
	s.insert( ipnet4t("255.255.255.255") );
	s.insert( ipnet4t("192.168.10.64/26") );
	s.insert( ipnet4t("0.0.0.0/30") );
	s.insert( ipnet4t("255.255.255.252/30") );
	s.insert( ipnet4t("0.0.0.0/0") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "0.0.0.0/0" )
	);
    }

    //---2------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.11") );
	s.insert( ipnet4t("192.168.10.8/29") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.8/29" )
	);
    }

    //---3------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/29") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.0/28" )
	);
    }

    //---4------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.11") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.8/29" )
	);
    }

    //---5------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.0/29") );
	s.insert( ipnet4t("192.168.10.8/29") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.0/28" )
	);
    }

    //---6------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.0/29") );
	s.insert( ipnet4t("192.168.10.8/30") );

	ipset4t::const_iterator p1( s.begin() );
	ipset4t::const_iterator p0( p1++ );

	t.check
	(
	    s.size() == 2 &&
	    *p0 == ipnet4t( "192.168.10.0/29" ) &&
	    *p1 == ipnet4t( "192.168.10.8/30" )
	);
    }

    //---7------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.8/30") );
	s.insert( ipnet4t("192.168.10.0/29") );

	ipset4t::const_iterator p1( s.begin() );
	ipset4t::const_iterator p0( p1++ );

	t.check
	(
	    s.size() == 2 &&
	    *p0 == ipnet4t( "192.168.10.0/29" ) &&
	    *p1 == ipnet4t( "192.168.10.8/30" )
	);
    }

    //---8------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("10.0.0.3") );
	s.insert( ipnet4t("10.0.0.1") );
	s.insert( ipnet4t("10.0.0.2") );
	s.insert( ipnet4t("10.0.0.0") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "10.0.0.0/30" )
	);
    }

    //---9------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("10.0.0.0") );
	s.insert( ipnet4t("10.0.0.2") );

	ipset4t::const_iterator p1( s.begin() );
	ipset4t::const_iterator p0( p1++ );

	t.check
	(
	    s.size() == 2 &&
	    *p0 == ipnet4t( "10.0.0.0" ) &&
	    *p1 == ipnet4t( "10.0.0.2" )
	);
    }

    //---10-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("0.0.0.0") );
	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.5") );
	s.insert( ipnet4t("0.0.0.3") );
	s.insert( ipnet4t("0.0.0.1") );

	ipset4t::const_iterator p1( s.begin() );
	ipset4t::const_iterator p0( p1++ );

	t.check
	(
	    s.size() == 2 &&
	    *p0 == ipnet4t( "0.0.0.0/30" ) &&
	    *p1 == ipnet4t( "0.0.0.5" )
	);
    }

    //---11-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("255.255.255.255") );
	s.insert( ipnet4t("255.255.255.250") );
	s.insert( ipnet4t("255.255.255.254") );
	s.insert( ipnet4t("255.255.255.252") );
	s.insert( ipnet4t("255.255.255.253") );

	ipset4t::const_iterator p1( s.begin() );
	ipset4t::const_iterator p0( p1++ );

	t.check
	(
	    s.size() == 2 &&
	    *p0 == ipnet4t( "255.255.255.250" ) &&
	    *p1 == ipnet4t( "255.255.255.252/30" )
	);
    }

    //---12-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.8/30") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.8/29" )
	);
    }

    //---13-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.8/30") );
	s.insert( ipnet4t("192.168.10.8/29") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.8/29" )
	);
    }

    //---13-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.4/30") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.0/28" )
	);
    }

    //---14-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.16/28") );
	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.4/30") );

	ipset4t::const_iterator p0( s.begin() );

	t.check
	(
	    s.size() == 1 &&
	    *p0 == ipnet4t( "192.168.10.0/27" )
	);
    }

    //---15-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.16") );
	s.insert( ipnet4t("192.168.10.8") );
	s.insert( ipnet4t("192.168.10.0") );
	s.insert( ipnet4t("192.168.10.4") );

	ipset4t::const_iterator p3( s.begin() );
	ipset4t::const_iterator p0( p3++ );
	ipset4t::const_iterator p1( p3++ );
	ipset4t::const_iterator p2( p3++ );

	t.check
	(
	    s.size() == 4 &&
	    *p0 == ipnet4t( "192.168.10.0" ) &&
	    *p1 == ipnet4t( "192.168.10.4" ) &&
	    *p2 == ipnet4t( "192.168.10.8" ) &&
	    *p3 == ipnet4t( "192.168.10.16" )
	);
    }

    //---16-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("81.17.162.64/29") );
	s.insert( ipnet4t("81.17.162.80/29") );
	s.insert( ipnet4t("81.17.162.88/29") );
	s.insert( ipnet4t("81.17.162.96/29") );
	s.insert( ipnet4t("81.17.162.104/30") );
	s.insert( ipnet4t("81.17.162.108/30") );

	ipset4t::const_iterator p2( s.begin() );
	ipset4t::const_iterator p0( p2++ );
	ipset4t::const_iterator p1( p2++ );

	t.check
	(
	    s.size() == 3 &&
	    *p0 == ipnet4t( "81.17.162.64/29" ) &&
	    *p1 == ipnet4t( "81.17.162.80/28" ) &&
	    *p2 == ipnet4t( "81.17.162.96/28" )
	);
    }

    return t.result();
}

/**************************************************************************************************************/
bool
SetBinarySearchTest()
{
    struct sett : public ipset4t
    {
	bool
	contains( ipaddr4t Ip ) const
	{
	    set<ipnet4t>::const_iterator  lb( lower_bound(_networks.begin(), _networks.end(), Ip) );
	    set<ipnet4t>::const_iterator  lbc( lb );

	    return (lb != _networks.begin() && (--lbc)->contains(Ip)) || (lb != _networks.end() && lb->contains(Ip));
	}

	bool
	contains( ipnet4t Net ) const
	{
	    set<ipnet4t>::const_iterator  lb( lower_bound(_networks.begin(), _networks.end(), Net) );
	    set<ipnet4t>::const_iterator  lbc( lb );

	    return (lb != _networks.begin() && (--lbc)->contains(Net)) || (lb != _networks.end() && lb->contains(Net));
	}
    };

    testt  t( "SetBinarySearchTest()" );

    //---1------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    s.contains( ipaddr4t("0.0.0.2") ) &&
	    !s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    s.contains( ipaddr4t("0.0.0.8") ) &&
	    !s.contains( ipaddr4t("0.0.0.9") )
	);
    }

    //---2------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.9") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    s.contains( ipaddr4t("0.0.0.2") ) &&
	    !s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    !s.contains( ipaddr4t("0.0.0.8") ) &&
	    s.contains( ipaddr4t("0.0.0.9") ) &&
	    !s.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---3------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.3") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.9") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    !s.contains( ipaddr4t("0.0.0.2") ) &&
	    s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    !s.contains( ipaddr4t("0.0.0.8") ) &&
	    s.contains( ipaddr4t("0.0.0.9") ) &&
	    !s.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---4------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.3") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    !s.contains( ipaddr4t("0.0.0.2") ) &&
	    s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    s.contains( ipaddr4t("0.0.0.8") ) &&
	    !s.contains( ipaddr4t("0.0.0.9") ) &&
	    !s.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---5------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("192.168.10.16/28") );
	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.32/30") );

	t.check
	(
	    !s.contains( ipnet4t("192.168.9.252/30") ) &&
	    s.contains( ipnet4t("192.168.10.0/30") ) &&
	    !s.contains( ipnet4t("192.168.10.0/29") ) &&
	    !s.contains( ipnet4t("192.168.10.4/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/29") ) &&
	    s.contains( ipnet4t("192.168.10.12/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/29") ) &&
	    s.contains( ipnet4t("192.168.10.16/28") ) &&
	    s.contains( ipnet4t("192.168.10.20/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/29") ) &&
	    s.contains( ipnet4t("192.168.10.28/30") ) &&
	    s.contains( ipnet4t("192.168.10.32/30") ) &&
	    !s.contains( ipnet4t("192.168.10.32/29") ) &&
	    !s.contains( ipnet4t("192.168.10.32/28") ) &&
	    !s.contains( ipnet4t("192.168.10.32/27") ) &&
	    !s.contains( ipnet4t("192.168.10.36/30") ) &&
	    !s.contains( ipnet4t("192.168.10.40/30") )
	);
    }

    //---6------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("192.168.10.16/28") );
	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.36/30") );

	t.check
	(
	    !s.contains( ipnet4t("192.168.9.252/30") ) &&
	    s.contains( ipnet4t("192.168.10.0/30") ) &&
	    !s.contains( ipnet4t("192.168.10.0/29") ) &&
	    !s.contains( ipnet4t("192.168.10.4/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/29") ) &&
	    s.contains( ipnet4t("192.168.10.12/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/29") ) &&
	    s.contains( ipnet4t("192.168.10.16/28") ) &&
	    s.contains( ipnet4t("192.168.10.20/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/29") ) &&
	    s.contains( ipnet4t("192.168.10.28/30") ) &&
	    !s.contains( ipnet4t("192.168.10.32/30") ) &&
	    !s.contains( ipnet4t("192.168.10.32/29") ) &&
	    !s.contains( ipnet4t("192.168.10.32/28") ) &&
	    !s.contains( ipnet4t("192.168.10.32/27") ) &&
	    s.contains( ipnet4t("192.168.10.36/30") ) &&
	    !s.contains( ipnet4t("192.168.10.40/30") )
	);
    }

    //---7------------------------------------------------------------------------------------------------------
    {
	sett  s;

	t.check
	(
	    s.empty() &&
	    !s.contains( ipnet4t("192.168.9.252/30") )
	);
    }

    return t.result();
}

/**************************************************************************************************************/
bool
SetLinearSearchTest()
{
    struct sett : public ipset4t
    {
	bool
	contains( ipaddr4t Ip ) const
	{
	    return
	    (
	        find_if
	        (
	            _networks.begin(),
	            _networks.end(),
	            bind2nd( mem_fun_ref((bool (ipnet4t::*)(ipaddr4t) const)&ipnet4t::contains), Ip )
	        )
	        != _networks.end()
	    );
	}

	bool
	contains( ipnet4t Net ) const
	{
	    return
	    (
	        find_if
	        (
	            _networks.begin(),
	            _networks.end(),
	            bind2nd( mem_fun_ref((bool (ipnet4t::*)(ipnet4t) const)&ipnet4t::contains), Net )
	        )
	        != _networks.end()
	    );
	}
    };

    testt  t( "SetLinearSearchTest()" );

    //---1------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    s.contains( ipaddr4t("0.0.0.2") ) &&
	    !s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    s.contains( ipaddr4t("0.0.0.8") ) &&
	    !s.contains( ipaddr4t("0.0.0.9") )
	);
    }

    //---2------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.9") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    s.contains( ipaddr4t("0.0.0.2") ) &&
	    !s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    !s.contains( ipaddr4t("0.0.0.8") ) &&
	    s.contains( ipaddr4t("0.0.0.9") ) &&
	    !s.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---3------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.3") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.9") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    !s.contains( ipaddr4t("0.0.0.2") ) &&
	    s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    !s.contains( ipaddr4t("0.0.0.8") ) &&
	    s.contains( ipaddr4t("0.0.0.9") ) &&
	    !s.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---4------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("0.0.0.3") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	t.check
	(
	    !s.contains( ipaddr4t("0.0.0.0") ) &&
	    !s.contains( ipaddr4t("0.0.0.1") ) &&
	    !s.contains( ipaddr4t("0.0.0.2") ) &&
	    s.contains( ipaddr4t("0.0.0.3") ) &&
	    s.contains( ipaddr4t("0.0.0.4") ) &&
	    s.contains( ipaddr4t("0.0.0.5") ) &&
	    s.contains( ipaddr4t("0.0.0.6") ) &&
	    s.contains( ipaddr4t("0.0.0.7") ) &&
	    s.contains( ipaddr4t("0.0.0.8") ) &&
	    !s.contains( ipaddr4t("0.0.0.9") ) &&
	    !s.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---5------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("192.168.10.16/28") );
	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.32/30") );

	t.check
	(
	    !s.contains( ipnet4t("192.168.9.252/30") ) &&
	    s.contains( ipnet4t("192.168.10.0/30") ) &&
	    !s.contains( ipnet4t("192.168.10.0/29") ) &&
	    !s.contains( ipnet4t("192.168.10.4/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/29") ) &&
	    s.contains( ipnet4t("192.168.10.12/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/29") ) &&
	    s.contains( ipnet4t("192.168.10.16/28") ) &&
	    s.contains( ipnet4t("192.168.10.20/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/29") ) &&
	    s.contains( ipnet4t("192.168.10.28/30") ) &&
	    s.contains( ipnet4t("192.168.10.32/30") ) &&
	    !s.contains( ipnet4t("192.168.10.32/29") ) &&
	    !s.contains( ipnet4t("192.168.10.32/28") ) &&
	    !s.contains( ipnet4t("192.168.10.32/27") ) &&
	    !s.contains( ipnet4t("192.168.10.36/30") ) &&
	    !s.contains( ipnet4t("192.168.10.40/30") )
	);
    }

    //---6------------------------------------------------------------------------------------------------------
    {
	sett  s;

	s.insert( ipnet4t("192.168.10.16/28") );
	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.36/30") );

	t.check
	(
	    !s.contains( ipnet4t("192.168.9.252/30") ) &&
	    s.contains( ipnet4t("192.168.10.0/30") ) &&
	    !s.contains( ipnet4t("192.168.10.0/29") ) &&
	    !s.contains( ipnet4t("192.168.10.4/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/30") ) &&
	    s.contains( ipnet4t("192.168.10.8/29") ) &&
	    s.contains( ipnet4t("192.168.10.12/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/30") ) &&
	    s.contains( ipnet4t("192.168.10.16/29") ) &&
	    s.contains( ipnet4t("192.168.10.16/28") ) &&
	    s.contains( ipnet4t("192.168.10.20/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/30") ) &&
	    s.contains( ipnet4t("192.168.10.24/29") ) &&
	    s.contains( ipnet4t("192.168.10.28/30") ) &&
	    !s.contains( ipnet4t("192.168.10.32/30") ) &&
	    !s.contains( ipnet4t("192.168.10.32/29") ) &&
	    !s.contains( ipnet4t("192.168.10.32/28") ) &&
	    !s.contains( ipnet4t("192.168.10.32/27") ) &&
	    s.contains( ipnet4t("192.168.10.36/30") ) &&
	    !s.contains( ipnet4t("192.168.10.40/30") )
	);
    }

    //---7------------------------------------------------------------------------------------------------------
    {
	sett  s;

	t.check
	(
	    s.empty() &&
	    !s.contains( ipnet4t("192.168.9.252/30") )
	);
    }

    return t.result();
}

/**************************************************************************************************************/
bool
LookupTest()
{
    testt  t( "LookupTest()" );

    //---1------------------------------------------------------------------------------------------------------
    try
    {
	ipset4t  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	iplookup4t  l( s, 0 );
    }
    catch ( range_error )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---2------------------------------------------------------------------------------------------------------
    try
    {
	ipset4t  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	iplookup4t  l( s, 21 );
    }
    catch ( range_error )
    {
    	t.check( true );
    }
    catch ( ... )
    {
	t.check( false );
    }
   
    //---3------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	iplookup4t  l( s );

	t.check
	(
	    !l.contains( ipaddr4t("0.0.0.0") ) &&
	    !l.contains( ipaddr4t("0.0.0.1") ) &&
	    l.contains( ipaddr4t("0.0.0.2") ) &&
	    !l.contains( ipaddr4t("0.0.0.3") ) &&
	    l.contains( ipaddr4t("0.0.0.4") ) &&
	    l.contains( ipaddr4t("0.0.0.5") ) &&
	    l.contains( ipaddr4t("0.0.0.6") ) &&
	    l.contains( ipaddr4t("0.0.0.7") ) &&
	    l.contains( ipaddr4t("0.0.0.8") ) &&
	    !l.contains( ipaddr4t("0.0.0.9") )
	);
    }

    //---4------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("0.0.0.2") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.9") );

	iplookup4t  l( s );

	t.check
	(
	    !l.contains( ipaddr4t("0.0.0.0") ) &&
	    !l.contains( ipaddr4t("0.0.0.1") ) &&
	    l.contains( ipaddr4t("0.0.0.2") ) &&
	    !l.contains( ipaddr4t("0.0.0.3") ) &&
	    l.contains( ipaddr4t("0.0.0.4") ) &&
	    l.contains( ipaddr4t("0.0.0.5") ) &&
	    l.contains( ipaddr4t("0.0.0.6") ) &&
	    l.contains( ipaddr4t("0.0.0.7") ) &&
	    !l.contains( ipaddr4t("0.0.0.8") ) &&
	    l.contains( ipaddr4t("0.0.0.9") ) &&
	    !l.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---5------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("0.0.0.3") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.9") );

	iplookup4t  l( s );

	t.check
	(
	    !l.contains( ipaddr4t("0.0.0.0") ) &&
	    !l.contains( ipaddr4t("0.0.0.1") ) &&
	    !l.contains( ipaddr4t("0.0.0.2") ) &&
	    l.contains( ipaddr4t("0.0.0.3") ) &&
	    l.contains( ipaddr4t("0.0.0.4") ) &&
	    l.contains( ipaddr4t("0.0.0.5") ) &&
	    l.contains( ipaddr4t("0.0.0.6") ) &&
	    l.contains( ipaddr4t("0.0.0.7") ) &&
	    !l.contains( ipaddr4t("0.0.0.8") ) &&
	    l.contains( ipaddr4t("0.0.0.9") ) &&
	    !l.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---6------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("0.0.0.3") );
	s.insert( ipnet4t("0.0.0.4/30") );
	s.insert( ipnet4t("0.0.0.8") );

	iplookup4t  l( s );

	t.check
	(
	    !l.contains( ipaddr4t("0.0.0.0") ) &&
	    !l.contains( ipaddr4t("0.0.0.1") ) &&
	    !l.contains( ipaddr4t("0.0.0.2") ) &&
	    l.contains( ipaddr4t("0.0.0.3") ) &&
	    l.contains( ipaddr4t("0.0.0.4") ) &&
	    l.contains( ipaddr4t("0.0.0.5") ) &&
	    l.contains( ipaddr4t("0.0.0.6") ) &&
	    l.contains( ipaddr4t("0.0.0.7") ) &&
	    l.contains( ipaddr4t("0.0.0.8") ) &&
	    !l.contains( ipaddr4t("0.0.0.9") ) &&
	    !l.contains( ipaddr4t("0.0.0.10") )
	);
    }

    //---7------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.16/28") );
	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.32/30") );

	iplookup4t  l( s );

	t.check
	(
	    !l.contains( ipnet4t("192.168.9.252/30") ) &&
	    l.contains( ipnet4t("192.168.10.0/30") ) &&
	    !l.contains( ipnet4t("192.168.10.0/29") ) &&
	    !l.contains( ipnet4t("192.168.10.4/30") ) &&
	    l.contains( ipnet4t("192.168.10.8/30") ) &&
	    l.contains( ipnet4t("192.168.10.8/29") ) &&
	    l.contains( ipnet4t("192.168.10.12/30") ) &&
	    l.contains( ipnet4t("192.168.10.16/30") ) &&
	    l.contains( ipnet4t("192.168.10.16/29") ) &&
	    l.contains( ipnet4t("192.168.10.16/28") ) &&
	    l.contains( ipnet4t("192.168.10.20/30") ) &&
	    l.contains( ipnet4t("192.168.10.24/30") ) &&
	    l.contains( ipnet4t("192.168.10.24/29") ) &&
	    l.contains( ipnet4t("192.168.10.28/30") ) &&
	    l.contains( ipnet4t("192.168.10.32/30") ) &&
	    !l.contains( ipnet4t("192.168.10.32/29") ) &&
	    !l.contains( ipnet4t("192.168.10.32/28") ) &&
	    !l.contains( ipnet4t("192.168.10.32/27") ) &&
	    !l.contains( ipnet4t("192.168.10.36/30") ) &&
	    !l.contains( ipnet4t("192.168.10.40/30") )
	);
    }

    //---8------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("192.168.10.16/28") );
	s.insert( ipnet4t("192.168.10.8/29") );
	s.insert( ipnet4t("192.168.10.0/30") );
	s.insert( ipnet4t("192.168.10.36/30") );

	iplookup4t  l( s );

	t.check
	(
	    !l.contains( ipnet4t("192.168.9.252/30") ) &&
	    l.contains( ipnet4t("192.168.10.0/30") ) &&
	    !l.contains( ipnet4t("192.168.10.0/29") ) &&
	    !l.contains( ipnet4t("192.168.10.4/30") ) &&
	    l.contains( ipnet4t("192.168.10.8/30") ) &&
	    l.contains( ipnet4t("192.168.10.8/29") ) &&
	    l.contains( ipnet4t("192.168.10.12/30") ) &&
	    l.contains( ipnet4t("192.168.10.16/30") ) &&
	    l.contains( ipnet4t("192.168.10.16/29") ) &&
	    l.contains( ipnet4t("192.168.10.16/28") ) &&
	    l.contains( ipnet4t("192.168.10.20/30") ) &&
	    l.contains( ipnet4t("192.168.10.24/30") ) &&
	    l.contains( ipnet4t("192.168.10.24/29") ) &&
	    l.contains( ipnet4t("192.168.10.28/30") ) &&
	    !l.contains( ipnet4t("192.168.10.32/30") ) &&
	    !l.contains( ipnet4t("192.168.10.32/29") ) &&
	    !l.contains( ipnet4t("192.168.10.32/28") ) &&
	    !l.contains( ipnet4t("192.168.10.32/27") ) &&
	    l.contains( ipnet4t("192.168.10.36/30") ) &&
	    !l.contains( ipnet4t("192.168.10.40/30") )
	);
    }

    //---9------------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	iplookup4t  l( s );

	t.check
	(
	    !l.contains( ipnet4t("192.168.9.252/30") )
	);
    }

    //---10-----------------------------------------------------------------------------------------------------
    {
	ipset4t  s;

	s.insert( ipnet4t("10/8") );

	iplookup4t  l( s );

	t.check
	(
	    l.contains( ipnet4t("10/8") ) &&
	    !l.contains( ipnet4t("10/7") ) &&
	    l.contains( ipnet4t("10.0/9") ) &&
	    l.contains( ipnet4t("10.128/9") ) &&
	    l.contains( ipnet4t("10.0/10") ) &&
	    l.contains( ipnet4t("10.0/11") ) &&
	    l.contains( ipnet4t("10.0/12") ) &&
	    l.contains( ipnet4t("10.16/12") ) &&
	    l.contains( ipnet4t("10.32/12") ) &&
	    l.contains( ipnet4t("10.240/12") ) &&
	    l.contains( ipnet4t("10.0/13") ) &&
	    l.contains( ipnet4t("10.0/14") ) &&
	    l.contains( ipnet4t("10.0/16") ) &&
	    l.contains( ipnet4t("10.255/16") ) &&
	    l.contains( ipaddr4t("10.0.0.0") ) &&
	    l.contains( ipaddr4t("10.0.0.1") ) &&
	    l.contains( ipaddr4t("10.255.255.255") ) &&
	    !l.contains( ipaddr4t("9.255.255.255") ) &&
	    !l.contains( ipaddr4t("11.0.0.0") ) &&
	    !l.contains( ipnet4t("9.255.255.252/30") ) &&
	    !l.contains( ipnet4t("11.0.0.0/30") )
	);
    }

    return t.result();
}

/**************************************************************************************************************/
int
main()
{
    NumericAddressTest();
    StringToAddressTest();
    AddressToStringTest();
    AddressComparisonTest();
    AddressOperatorsTest();
    NumericNetworkTest();
    StringToNetworkTest();
    NetworkToStringTest();
    NetworkComparisonTest();
    NetworkInclusionTest();
    NetworkAdjacentTest();
    NetworkMergeableTest();
    NetworkMergeTest();
    SetInsertTest();
    SetBinarySearchTest();
    SetLinearSearchTest();
    LookupTest();

    return 0;
}
