/* sizeof.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <cmath>
#include <cstring>
#include <ctime>
#include <iostream>
#include <limits>
#include <sstream>
#include <string>
#include <libiclass/task>
#include <libiclass/time>
#include <stdint.h>

using namespace std;
using namespace iclass;

/**************************************************************************************************************/
int
main()
{
    cout << "sizeof(long double)        = " << sizeof(long double)
	<< "; min = " << numeric_limits<long double>::min()
	<< "; max = " << numeric_limits<long double>::max() << endl;
	
    cout << "sizeof(double)             = " << sizeof(double)
	<< "; min = " << numeric_limits<double>::min()
	<< "; max = " << numeric_limits<double>::max() << endl;
	
    cout << "sizeof(float)              = " << sizeof(float)
	<< "; min = " << numeric_limits<float>::min()
	<< "; max = " << numeric_limits<float>::max() << endl;
	
    cout << "sizeof(long long)          = " << sizeof(long long)
	<< "; min = " << numeric_limits<long long>::min()
	<< "; max = " << numeric_limits<long long>::max() << endl;
	
    cout << "sizeof(unsigned long long) = " << sizeof(unsigned long long)
	<< "; min = " << numeric_limits<unsigned long long>::min()
	<< "; max = " << numeric_limits<unsigned long long>::max() << endl;
	
    cout << "sizeof(long)               = " << sizeof(long)
	<< "; min = " << numeric_limits<long>::min()
	<< "; max = " << numeric_limits<long>::max() << endl;
	
    cout << "sizeof(int)                = " << sizeof(int)
	<< "; min = " << numeric_limits<int>::min()
	<< "; max = " << numeric_limits<int>::max() << endl;
	
    cout << "sizeof(short)              = " << sizeof(short)
	<< "; min = " << numeric_limits<short>::min()
	<< "; max = " << numeric_limits<short>::max() << endl;
	
    cout << "sizeof(unsigned)           = " << sizeof(unsigned)
	<< "; min = " << numeric_limits<unsigned>::min()
	<< "; max = " << numeric_limits<unsigned>::max() << endl;
	
    cout << "sizeof(char)               = " << sizeof(char)
	<< "; min = " << static_cast<int>( numeric_limits<char>::min() )
	<< "; max = " << static_cast<int>( numeric_limits<char>::max() ) << endl;
	
    cout << "sizeof(unsigned char)      = " << sizeof(unsigned char)
	<< "; min = " << static_cast<int>( numeric_limits<unsigned char>::min() )
	<< "; max = " << static_cast<int>( numeric_limits<unsigned char>::max() ) << endl;
	
    cout << "sizeof(bool)               = " << sizeof(bool)
	<< "; min = " << static_cast<int>( numeric_limits<bool>::min() )
	<< "; max = " << static_cast<int>( numeric_limits<bool>::max() ) << endl;
	
    cout << "sizeof(uint16_t)           = " << sizeof(uint16_t)
	<< "; min = " << numeric_limits<uint16_t>::min()
	<< "; max = " << numeric_limits<uint16_t>::max() << endl;
	
    cout << "sizeof(uint32_t)           = " << sizeof(uint32_t)
	<< "; min = " << numeric_limits<uint32_t>::min()
	<< "; max = " << numeric_limits<uint32_t>::max() << endl;
	
    time_t  mintm( numeric_limits<time_t>::min() );
    time_t  maxtm( numeric_limits<time_t>::max() );
    gtimet  mingtime( mintm );
    gtimet  maxgtime( maxtm );

    cout << "sizeof(time_t)             = " << sizeof(time_t)
	<< "; min = " << mintm
	<< "; max = " << maxtm
	<< "; [ " << mingtime.str() << "; " << maxgtime.str() << " ]" << endl;
	
    cout << "sizeof(size_t)             = " << sizeof(size_t)
	<< "; min = " << numeric_limits<size_t>::min()
	<< "; max = " << numeric_limits<size_t>::max() << endl;
	
    cout << "sizeof(string::size_type)  = " << sizeof(string::size_type)
	<< "; min = " << numeric_limits<string::size_type>::min()
	<< "; max = " << numeric_limits<string::size_type>::max() << endl;
    
    const char*	 anystr = "anystring";

    cout << endl << "anystr = " << anystr << endl;
    cout << "sizeof(anystr)             = " << sizeof(anystr) << endl;
    cout << "strlen(anystr)             = " << strlen(anystr) << endl;

    int	 mib[] = { 0, 1, 2, 3 };

    cout << endl << "mib[] = { " << mib[0] << ", " << mib[1] << ", " << mib[2] << ", " << mib[3] << " }" << endl;
    cout << "sizeof(mib)                = " << sizeof(mib) << endl;
    cout << "sizeof(mib[0])             = " << sizeof(mib[0]) << endl << endl;

    cout << "Scheduler priority ranges" << endl;

    int  schedother_min( sched_get_priority_min(SCHED_OTHER) );
    int  schedother_max( sched_get_priority_max(SCHED_OTHER) );

    cout << "SCHED_OTHER (" << SCHED_OTHER << "):\t   ";

    if ( schedother_min != -1 )
    {
	cout << "[ " << schedother_min << "; " << schedother_max << " ]" << endl;
    }
    else
    {
	cout << "unsupported" << endl;
    }

    int  schedrr_min( sched_get_priority_min(SCHED_RR) );
    int  schedrr_max( sched_get_priority_max(SCHED_RR) );

    cout << "SCHED_RR (" << SCHED_RR << "):\t\t   ";

    if ( schedrr_min != -1 )
    {
	cout << "[ " << schedrr_min << "; " << schedrr_max << " ]" << endl;
    }
    else
    {
	cout << "unsupported" << endl;
    }

    int  schedfifo_min( sched_get_priority_min(SCHED_FIFO) );
    int  schedfifo_max( sched_get_priority_max(SCHED_FIFO) );

    cout << "SCHED_FIFO (" << SCHED_FIFO << "):\t\t   ";

    if ( schedfifo_min != -1 )
    {
	cout << "[ " << schedfifo_min << "; " << schedfifo_max << " ]" << endl;
    }
    else
    {
	cout << "unsupported" << endl;
    }

    priorityt  mainprio;

    cout << "Default:\t\t   " << mainprio.policyname() << ":" << mainprio.value() << endl;

    return 0;    
}

