/* tasktest.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <cstdlib>
#include <iostream>
#include <unistd.h>
#include "tasktest.h"

using namespace iclass;
using namespace std;

countert   countertaskt::_c;
pcountert  countertaskt::_pc;
int        srwtaskt::_shared( 1 );

/**************************************************************************************************************/
static priorityt
_RandPrio( int Policy )
{
    priorityt  minprio( priorityt::min(Policy) );
    priorityt  maxprio( priorityt::max(Policy) );

    if ( maxprio == minprio )
    {
	return minprio;
    }

    return minprio + random() % (maxprio - minprio);
}

/**************************************************************************************************************/
bool
SchedulerTest()
{
    cout << "SchedulerTest() ... " << flush;

    int	       pid( getpid() );
    threadt    th;
    priorityt  mainprio;		// current thread priority
    priorityt  sampleprio( priorityt::max(SCHED_RR) - 15 );
    priorityt  prio;

    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( mainprio.policy() == SCHED_OTHER );

    prio = sampleprio + 2;
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio + 2 );

    prio -= 3;
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() - sampleprio == -1 );
    assert( th.prio().policy() == SCHED_RR );

    th.prio( prio++ );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio - 1 );

    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio );

    th.prio( ++prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio + 1 );

    th.prio( prio-- );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio + 1 );

    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio );

    th.prio( --prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio - 1 );

    prio += 3;
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() - sampleprio == 2 );
    assert( th.prio().policy() == SCHED_RR );

    prio = sampleprio - 2;
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == sampleprio - 2 );

    prio = priorityt( SCHED_OTHER, mainprio.value() );
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = prio.minvalue();
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = prio.maxvalue();
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = priorityt( SCHED_RR, sampleprio.value() );
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = prio.minvalue();
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = prio.maxvalue();
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = priorityt( SCHED_FIFO, sampleprio.value() );
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = prio.minvalue();
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    prio = prio.maxvalue();
    th.prio( prio );
    //cout << "pid=" << pid << "\ttid=" << th.tid() << "\tprio=" << th.prio().value() << "\tpolicy=" << th.prio().policy() << endl;
    assert( th.prio() == prio );

    th.prio( priorityt::min(SCHED_RR) + 2 );
    assert( th.prio() - priorityt::min(SCHED_RR) == 2 );

    th.prio( priorityt::max(SCHED_RR) - 2 );
    assert( priorityt::max(SCHED_RR) - th.prio() == 2 );

    th.prio( priorityt::min(SCHED_FIFO) + 2 );
    assert( th.prio() - priorityt::min(SCHED_FIFO) == 2 );

    th.prio( priorityt::max(SCHED_FIFO) - 2 );
    assert( priorityt::max(SCHED_FIFO) - th.prio() == 2 );

    th.prio( mainprio );

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
CommonTest()
{
    countert  common;

    cout << "CommonTest() ... " << flush;

    common.lock();
    common.lock();
    common.unlock();
    common.unlock();
    common.lock();

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
QuicktaskTest( int Policy )
{
    threadt	 th;
    priorityt    mainprio;		// current thread priority
    priorityt	 prio( priorityt::max(Policy) );
    quicktaskt*  qtarray[ 20 ];
    int          i;

    cout << "QuicktaskTest( " << prio.policyname() << " ) ... " << flush;

    th.prio( prio );			// set maximum priority to launcher

    for ( i = 0; i < 20; i++ )
    {
        qtarray[ i ] = new quicktaskt;
        qtarray[ i ]->start( _RandPrio(Policy) );
    }

    for ( int j = 0; j < 1000; j++ )
    {
        i = random() % 20;

        qtarray[ i ]->stop();
        delete qtarray[ i ];
        qtarray[ i ] = new quicktaskt;
        qtarray[ i ]->start( _RandPrio(Policy) );
    }

    for ( i = 0; i < 20; i++ )
    {
        qtarray[ i ]->stop();
        delete qtarray[ i ];
    }

    th.prio( mainprio );		// return to default priority

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
SlowtaskTest( int Policy )
{
    threadt	th;
    priorityt   mainprio;		// current thread priority
    priorityt	prio( priorityt::max(Policy) );
    slowtaskt*  starray[ 5 ];
    int         i;

    cout << "SlowtaskTest( " << prio.policyname() << " ) ... " << flush;

    th.prio( prio );			// set maximum priority to launcher

    for ( i = 0; i < 5; i++ )
    {
        starray[ i ] = new slowtaskt;
        starray[ i ]->start( _RandPrio(Policy) );
    }

    for ( int j = 0; j < 200; j++ )
    {
        i = random() % 5;

        starray[ i ]->stop();
        delete starray[ i ];
        starray[ i ] = new slowtaskt;
        starray[ i ]->start( _RandPrio(Policy) );
    }

    for ( i = 0; i < 5; i++ )
    {
        starray[ i ]->stop();
        delete starray[ i ];
    }

    th.prio( mainprio );		// return to default priority

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
StartStopQuicktaskTest( int Policy )
{
    threadt	th;
    priorityt   mainprio;		// current thread priority
    priorityt	prio( priorityt::max(Policy) );
    quicktaskt  qt;

    cout << "StartstopQuicktaskTest( " << prio.policyname() << " ) ... " << flush;

    th.prio( prio );			// set maximum priority to launcher

    qt.stop();
    assert( !qt.started() );		// false
    assert( !qt.stopped() );		// false, stop() ignored
    assert( !qt.finished() );		// false

    qt.start( _RandPrio(Policy) );
    assert( qt.started() );		// true
    assert( !qt.stopped() );		// false

    qt.start( _RandPrio(Policy) );
    assert( qt.started() );		// true
    assert( !qt.stopped() );		// false
    
    qt.stop();
    assert( qt.started() );		// true
    assert( qt.stopped() );		// true
    assert( qt.finished() );		// true

    qt.stop();
    assert( qt.started() );		// true
    assert( qt.stopped() );		// true
    assert( qt.finished() );		// true

    qt.start( _RandPrio(Policy) );
    assert( qt.started() );		// true
    assert( qt.stopped() );		// true
    assert( qt.finished() );		// true

    qt.stop();
    assert( qt.started() );		// true
    assert( qt.stopped() );		// true
    assert( qt.finished() );		// true

    th.prio( mainprio );		// return to default priority

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
StartStopSlowtaskTest( int Policy )
{
    threadt    th;
    priorityt  mainprio;		// current thread priority
    priorityt  prio( priorityt::max(Policy) );
    slowtaskt  st;

    cout << "StartstopSlowtaskTest( " << prio.policyname() << " ) ... " << flush;

    th.prio( prio );			// set maximum priority to launcher

    st.stop();
    assert( !st.started() );		// false
    assert( !st.stopped() );		// true
    assert( !st.finished() );		// false

    st.start( _RandPrio(Policy) );
    assert( st.started() );		// true
    assert( !st.stopped() );		// false

    st.start( _RandPrio(Policy) );
    assert( st.started() );		// true
    assert( !st.stopped() );		// false
    
    st.stop();
    assert( st.started() );		// true
    assert( st.stopped() );		// true
    assert( st.finished() );		// true

    st.stop();
    assert( st.started() );		// true
    assert( st.stopped() );		// true
    assert( st.finished() );		// true

    st.start( _RandPrio(Policy) );
    assert( st.started() );		// true
    assert( st.stopped() );		// true
    assert( st.finished() );		// true

    st.stop();
    assert( st.started() );		// true
    assert( st.stopped() );		// true
    assert( st.finished() );		// true

    th.prio( mainprio );		// return to default priority

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
CounterTest( int Policy )
{
    threadt	   th;
    priorityt      mainprio;		// current thread priority
    priorityt      prio( priorityt::max(Policy) );
    countertaskt*  ctarray[ 20 ];
    int            i;

    cout << "CounterTest( " << prio.policyname() << " ) ... " << flush;

    th.prio( prio );			// set maximum priority to launcher

    for ( i = 0; i < 20; i++ )
    {
        ctarray[ i ] = new countertaskt;
        ctarray[ i ]->start( _RandPrio(Policy) );
    }

    for ( int j = 0; j < 1000; j++ )
    {
        i = random() % 20;

        ctarray[ i ]->stop();
        delete ctarray[ i ];
        ctarray[ i ] = new countertaskt;
        ctarray[ i ]->start( _RandPrio(Policy) );
    }

    for ( i = 0; i < 20; i++ )
    {
        ctarray[ i ]->stop();
        delete ctarray[ i ];
    }

    th.prio( mainprio );		// return to default priority

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
SharedReadingWritingTest( int Policy )
{
    threadt    th;
    priorityt  mainprio;		// current thread priority
    priorityt  prio( priorityt::max(Policy) );
    srwtaskt*  srwtarray[ 5 ];
    int        i;

    cout << "SharedReadingWritingTest( " << prio.policyname() << " ) ... " << flush;

    th.prio( prio );			// set maximum priority to launcher

    for ( i = 0; i < 5; i++ )
    {
        srwtarray[ i ] = new srwtaskt;
        srwtarray[ i ]->start( _RandPrio(Policy) );
    }

    for ( int j = 0; j < 200; j++ )
    {
        i = random() % 5;

        srwtarray[ i ]->stop();
        delete srwtarray[ i ];
        srwtarray[ i ] = new srwtaskt;
        srwtarray[ i ]->start( _RandPrio(Policy) );
    }

    for ( i = 0; i < 5; i++ )
    {
        srwtarray[ i ]->stop();
        delete srwtarray[ i ];
    }

    th.prio( mainprio );		// return to default priority

    cout << "OK" << endl;

    return true;
}

/**************************************************************************************************************/
bool
CollectionTest()
{
    threadt	 th;
    priorityt    mainprio;		// current thread priority
    priorityt    prio( priorityt::max(SCHED_FIFO) );
    collectiont	 collection;
    collectort	 collector( collection );
    extractort	 extractor( collection );
    watchert	 watcher( collection );

    cout << "CollectionTest() " << flush;
    th.prio( prio );

    watcher.start( mainprio );
    extractor.start( mainprio );
    collector.start( mainprio );

    for ( int i=0; i<6; i++ )
    {
    	usleep( 500000 );

	cout << "." << flush;
    }

    collector.stop();
    extractor.stop();
    watcher.stop();
    
    th.prio( mainprio );		// return to default priority

    cout << (extractor.result()?" OK":" fail") << endl;

    return true;
}

/**************************************************************************************************************/
int
main()
{
    srandom( 500 );
    
    SchedulerTest();
    CommonTest();

    QuicktaskTest( SCHED_OTHER );
    QuicktaskTest( SCHED_RR );
    QuicktaskTest( SCHED_FIFO );

    SlowtaskTest( SCHED_OTHER );
    SlowtaskTest( SCHED_RR );
    SlowtaskTest( SCHED_FIFO );

    StartStopQuicktaskTest( SCHED_OTHER );
    StartStopQuicktaskTest( SCHED_RR );
    StartStopQuicktaskTest( SCHED_FIFO );

    StartStopSlowtaskTest( SCHED_OTHER );
    StartStopSlowtaskTest( SCHED_RR );
    StartStopSlowtaskTest( SCHED_FIFO );

    CounterTest( SCHED_OTHER );
    CounterTest( SCHED_RR );
    CounterTest( SCHED_FIFO );

    SharedReadingWritingTest( SCHED_OTHER );
    SharedReadingWritingTest( SCHED_RR );
    SharedReadingWritingTest( SCHED_FIFO );

    CollectionTest();

    return 0;
}
