/* tasktest.h
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <libiclass/common>
#include <libiclass/task>
#include <cassert>
#include <map>

/**************************************************************************************************************/
struct quicktaskt : public iclass::taskt
{
    //----------------------------------------------------------------------------------------------------------
    quicktaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    ~quicktaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    virtual void
    routine()
    {
    }
};

/**************************************************************************************************************/
struct slowtaskt : public iclass::taskt
{
    //----------------------------------------------------------------------------------------------------------
    slowtaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    ~slowtaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    virtual void
    routine()
    {
	while ( !stopped() )
	{
	    usleep( 1 );
	}
    }
};

/**************************************************************************************************************/
struct countert : public iclass::commont
{
    friend struct pcountert;

    //----------------------------------------------------------------------------------------------------------
    countert()
    {
	_cnt = 0;
    }

    //----------------------------------------------------------------------------------------------------------
    virtual
    ~countert()
    {
	this->lock();
    }

    //----------------------------------------------------------------------------------------------------------
    void
    increment()
    {
	lock();
	_cnt++;
	unlock();
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    unsigned  _cnt;
};

/**************************************************************************************************************/
struct pcountert : public iclass::commont
{
    //----------------------------------------------------------------------------------------------------------
    pcountert()
    {
	_pcnt = new countert;
    }

    //----------------------------------------------------------------------------------------------------------
    virtual
    ~pcountert()
    {
	lock();
	delete _pcnt;
    }

    //----------------------------------------------------------------------------------------------------------
    void
    change()
    { 
    	lock();
    	delete _pcnt;
    	_pcnt = new countert; 
    	unlock();
    }
    
    //----------------------------------------------------------------------------------------------------------
    void
    cincrement()
    { 
    	lock();
    	_pcnt->increment(); 
    	unlock();
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    countert*  _pcnt;
};

/**************************************************************************************************************/
struct countertaskt : public iclass::taskt
{
    //----------------------------------------------------------------------------------------------------------
    countertaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    ~countertaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    virtual void
    routine()
    {
    	_c.increment();
    	_pc.cincrement();
    	_pc.change();
    	_pc.cincrement();
    	_pc.change();
    	_pc.change();
    	_pc.cincrement();
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    static countert   _c;
    static pcountert  _pc;
};

/**************************************************************************************************************/
struct srwtaskt : public iclass::taskt
{
    //----------------------------------------------------------------------------------------------------------
    srwtaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    ~srwtaskt()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    virtual void
    routine()
    {
    	unsigned  count( 0 );

    	for ( ;; )
    	{
            count += _shared;
            _shared++;
            _shared = 1;

	    if ( stopped() )
	    {
	    	break;
	    }
	    else
	    {
		usleep( 1 );
	    }
    	}
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    static int  _shared;
};

/**************************************************************************************************************/
struct collectiont : public iclass::commont
{
    //----------------------------------------------------------------------------------------------------------
    collectiont()
    {
	_map[ -1 ] = -1;
    }

    //----------------------------------------------------------------------------------------------------------
    void
    push( int Value )
    { 
    	lock();
	_map[ Value ] = Value;
    	unlock();
    }
    
    //----------------------------------------------------------------------------------------------------------
    int
    pop()
    {
	lock();

	int  ret( 0 );

	if ( !_map.empty() )
	{
	    ret = _map.begin()->second;
	    _map.erase( _map.begin() );
	}

    	unlock();

	return ret;
    }

    //----------------------------------------------------------------------------------------------------------
    long
    size() const
    {
	lock();

	long  ret( _map.size() );

	unlock();

	return ret;
    }

    //----------------------------------------------------------------------------------------------------------
    bool
    empty() const
    {
	lock();

	long  ret( _map.empty() );

	unlock();

	return ret;
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    std::map<int,int>  _map;
};

/**************************************************************************************************************/
struct collectort : public iclass::taskt
{
    //----------------------------------------------------------------------------------------------------------
    collectort( collectiont& C ) :
	_collection( C )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    ~collectort()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    virtual void
    routine()
    {
    	for ( int i=1; i>0; i++ )
    	{
	    _collection.push( i );

	    if ( stopped() )
	    {
	    	break;
	    }
    	}
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    collectiont&  _collection;
};

/**************************************************************************************************************/
struct extractort : public iclass::taskt
{
    //----------------------------------------------------------------------------------------------------------
    extractort( collectiont& C ) :
	_collection( C ),
	_testresult( true )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    ~extractort()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    bool
    result() const
    {
	return _testresult;
    }

    //----------------------------------------------------------------------------------------------------------
    virtual void
    routine()
    {
	int checki( 1 );

    	while ( !stopped() )
    	{
	    if ( _collection.size() > 1 )
	    {
	    	int i( _collection.pop() );

	    	if ( i > 0 )
	    	{
		    if ( i != checki )
		    {
		    	_testresult = false;
		    }

		    checki++;
		}
	    }
    	}
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    collectiont&  _collection;
    bool	  _testresult;
};

/**************************************************************************************************************/
struct watchert : public iclass::taskt
{
    //----------------------------------------------------------------------------------------------------------
    watchert( collectiont& C ) :
	_collection( C )
    {
    }

    //----------------------------------------------------------------------------------------------------------
    ~watchert()
    {
    }

    //----------------------------------------------------------------------------------------------------------
    virtual void
    routine()
    {
    	for ( ;; )
    	{
	    assert( !_collection.empty() );

	    if ( stopped() )
	    {
	    	break;
	    }
    	}
    }

protected:

    //----------------------------------------------------------------------------------------------------------
    collectiont&  _collection;
};

