/* timetest.cc
****************************************************************************************************************
****************************************************************************************************************

    Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 Askar Almatov

    This file is part of Libiclass. 
    
    Libiclass is free software: you can redistribute it and/or modify it under the terms of the GNU General
    Public License as published by the Free Software Foundation, either version 3 of the License, or (at your
    option) any later version.
    
    Libiclass is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
    License for more details.
    
    You should have received a copy of the GNU General Public License along with Libiclass. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include <cstdlib>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <ctime>
#include <libiclass/period>
#include "test.h"
#include "timetest.h"

using namespace std;
using namespace iclass;
using namespace iclasstest;

/**************************************************************************************************************/
bool
TimezoneTest()
{
    testt   t( "TimezoneTest()" );
    ltimet  tm( 0 );
    string  pwd( string(":") + getenv("PWD") + "/" );
    string  pzi( "/usr/share/zoneinfo/" );

    if ( getenv("TZDIR") )
    {
	pzi = getenv( "TZDIR" );
	pzi += "/";
    }

    //--- 1 ----------------------------------------------------------------------------------------------------
    tm.settz( ":UTC" );
    t.check( tm.tz() == pwd + "UTC" );


    //--- 2 ----------------------------------------------------------------------------------------------------
    t.check( tm.islstimezone() );

    //--- 3 ----------------------------------------------------------------------------------------------------
    tm.settz( ":Almaty" );
    t.check( tm.tz() == pwd + "Almaty" );

    //--- 4 ----------------------------------------------------------------------------------------------------
    t.check( !tm.islstimezone() );

    //--- 5 ----------------------------------------------------------------------------------------------------
    tm.settz( ":Asia/Almaty" );
    t.check( tm.tz() == ":Asia/Almaty" );

    //--- 6 ----------------------------------------------------------------------------------------------------
    t.check( !tm.islstimezone() );

    //--- 7 ----------------------------------------------------------------------------------------------------
    tm.settz( ":Almaty_r" );
    t.check( tm.tz() == pwd + "Almaty_r" );

    //--- 8 ----------------------------------------------------------------------------------------------------
    t.check( tm.islstimezone() );

    //--- 9 ----------------------------------------------------------------------------------------------------
    tm.settz( ":Almaty_r " );
    t.check( tm.tz() == pwd + "Almaty_r" );

    //--- 10 ---------------------------------------------------------------------------------------------------
    t.check( tm.islstimezone() );

    //--- 11 ---------------------------------------------------------------------------------------------------
    tm.settz( ":./Almaty_r" );
    t.check( tm.tz() == pwd + "./Almaty_r" );

    //--- 12 ---------------------------------------------------------------------------------------------------
    t.check( tm.islstimezone() );

    //--- 13 ---------------------------------------------------------------------------------------------------
    tm.settz( "   :./Almaty_r" );
    t.check( tm.tz() == pwd + "./Almaty_r" );

    //--- 14 ---------------------------------------------------------------------------------------------------
    t.check( tm.islstimezone() );

    //--- 15 ---------------------------------------------------------------------------------------------------
    tm.settz( pzi + "Asia/Almaty" );
    t.check( tm.tz() == pzi + "Asia/Almaty" );

    //--- 16 ---------------------------------------------------------------------------------------------------
    t.check( !tm.islstimezone() );

    //--- 17 ---------------------------------------------------------------------------------------------------
    tm.settz( "AL -7:00" );
    t.check( tm.tz() == "AL -7:00" );

    //--- 18 ---------------------------------------------------------------------------------------------------
    t.check( !tm.islstimezone() );

    //--- 19 ---------------------------------------------------------------------------------------------------
    tm.settz( "   " );
    t.check( tm.tz().size() == 0 );

#if !defined( __MACH__ ) && !defined( __FreeBSD__ )

    //--- 20 ---------------------------------------------------------------------------------------------------
    tm.settz( ":right/Asia/Almaty" );
    t.check( tm.tz() == ":right/Asia/Almaty" );

    //--- 21 ---------------------------------------------------------------------------------------------------
    t.check( tm.islstimezone() );

    //--- 22 ---------------------------------------------------------------------------------------------------
    tm.settz( ":./right/Asia/Almaty" );
    t.check( tm.tz() == ":./right/Asia/Almaty" );

    //--- 23 ---------------------------------------------------------------------------------------------------
    t.check( tm.islstimezone() );

#endif

    return t.result();
}

/**************************************************************************************************************/
bool
TZLeapSecondTest( string Tz )
{
    ltimet  tm( 0 );

    tm.settz( Tz );

    struct tm  tma[] =
    {
	{ 59, 59, 23, 30,  6-1,  72,  0,  0,  -1 },	// 1
	{  0,  0,  0,  1,  7-1,  72,  0,  0,  -1 },	// 2
	{ 59, 59, 23, 31, 12-1,  72,  0,  0,  -1 },	// 3
	{  0,  0,  0,  1,  1-1,  73,  0,  0,  -1 },	// 4
	{ 59, 59, 23, 31, 12-1,  73,  0,  0,  -1 },	// 5
	{  0,  0,  0,  1,  1-1,  74,  0,  0,  -1 },	// 6
	{ 59, 59, 23, 31, 12-1,  74,  0,  0,  -1 },	// 7
	{  0,  0,  0,  1,  1-1,  75,  0,  0,  -1 },	// 8
	{ 59, 59, 23, 31, 12-1,  75,  0,  0,  -1 },	// 9
	{  0,  0,  0,  1,  1-1,  76,  0,  0,  -1 },	// 10
	{ 59, 59, 23, 31, 12-1,  76,  0,  0,  -1 },	// 11
	{  0,  0,  0,  1,  1-1,  77,  0,  0,  -1 },	// 12
	{ 59, 59, 23, 31, 12-1,  77,  0,  0,  -1 },	// 13
	{  0,  0,  0,  1,  1-1,  78,  0,  0,  -1 },	// 14
	{ 59, 59, 23, 31, 12-1,  78,  0,  0,  -1 },	// 15
	{  0,  0,  0,  1,  1-1,  79,  0,  0,  -1 },	// 16
	{ 59, 59, 23, 31, 12-1,  79,  0,  0,  -1 },	// 17
	{  0,  0,  0,  1,  1-1,  80,  0,  0,  -1 },	// 18
	{ 59, 59, 23, 30,  6-1,  81,  0,  0,  -1 },	// 19
	{  0,  0,  0,  1,  7-1,  81,  0,  0,  -1 },	// 20
	{ 59, 59, 23, 30,  6-1,  82,  0,  0,  -1 },	// 21
	{  0,  0,  0,  1,  7-1,  82,  0,  0,  -1 },	// 22
	{ 59, 59, 23, 30,  6-1,  83,  0,  0,  -1 },	// 23
	{  0,  0,  0,  1,  7-1,  83,  0,  0,  -1 },	// 24
	{ 59, 59, 23, 30,  6-1,  85,  0,  0,  -1 },	// 25
	{  0,  0,  0,  1,  7-1,  85,  0,  0,  -1 },	// 26
	{ 59, 59, 23, 31, 12-1,  87,  0,  0,  -1 },	// 27
	{  0,  0,  0,  1,  1-1,  88,  0,  0,  -1 },	// 28
	{ 59, 59, 23, 31, 12-1,  89,  0,  0,  -1 },	// 29
	{  0,  0,  0,  1,  1-1,  90,  0,  0,  -1 },	// 30
	{ 59, 59, 23, 31, 12-1,  90,  0,  0,  -1 },	// 31
	{  0,  0,  0,  1,  1-1,  91,  0,  0,  -1 },	// 32
	{ 59, 59, 23, 30,  6-1,  92,  0,  0,  -1 },	// 33
	{  0,  0,  0,  1,  7-1,  92,  0,  0,  -1 },	// 34
	{ 59, 59, 23, 30,  6-1,  93,  0,  0,  -1 },	// 35
	{  0,  0,  0,  1,  7-1,  93,  0,  0,  -1 },	// 36
	{ 59, 59, 23, 30,  6-1,  94,  0,  0,  -1 },	// 37
	{  0,  0,  0,  1,  7-1,  94,  0,  0,  -1 },	// 38
	{ 59, 59, 23, 31, 12-1,  95,  0,  0,  -1 },	// 39
	{  0,  0,  0,  1,  1-1,  96,  0,  0,  -1 },	// 40
	{ 59, 59, 23, 30,  6-1,  97,  0,  0,  -1 },	// 41
	{  0,  0,  0,  1,  7-1,  97,  0,  0,  -1 },	// 42
	{ 59, 59, 23, 31, 12-1,  98,  0,  0,  -1 },	// 43
	{  0,  0,  0,  1,  1-1,  99,  0,  0,  -1 },	// 44
	{ 59, 59, 23, 31, 12-1, 105,  0,  0,  -1 },	// 45
	{  0,  0,  0,  1,  1-1, 106,  0,  0,  -1 },	// 46
	{ 59, 59, 23, 31, 12-1, 108,  0,  0,  -1 },	// 47
	{  0,  0,  0,  1,  1-1, 109,  0,  0,  -1 },	// 48
	{ 59, 59, 23, 30,  6-1, 112,  0,  0,  -1 },	// 49
	{  0,  0,  0,  1,  7-1, 112,  0,  0,  -1 }	// 50
    };

    testt       t( string("TZLeapSecondTest( ") + tm.tz() + " )" );
    struct tm*  ptma( tma );

    for ( int i(0); i < sizeof(tma)/sizeof(struct tm); i++ )
    {
	tm = ltimet( mktime(ptma) );
	t.check( mktime(ptma) == tm.unixtime() );
	ptma++;
    }

    return t.result();
}

/**************************************************************************************************************/
bool
LeapSecondTest()
{
    testt      t( "LeapSecondTest()" );
    timetestt  tm;
    FILE*      tmpfile;
    
    tmpfile = fopen( "leapseconds.tmp", "w" );

    for ( int y(2013); y > 1972; y-- )
    {
	tm.setgall( y, 0, 0, 0, 0, 1 ); tm.debugout(tmpfile);
	tm.setgall( y, 0, 0, 0, 0, 0 ); tm.debugout(tmpfile);
    	tm.setgall( y-1, 11, 30, 23, 59, 60 ); tm.debugout(tmpfile);
    	tm.setgall( y-1, 11, 30, 23, 59, 60.01 ); tm.debugout(tmpfile);
    	tm.setgall( y-1, 11, 30, 23, 59, 59 ); tm.debugout(tmpfile);

	tm.setgall( y-1, 6, 0, 0, 0, 1 ); tm.debugout(tmpfile);
	tm.setgall( y-1, 6, 0, 0, 0, 0 ); tm.debugout(tmpfile);
    	tm.setgall( y-1, 5, 29, 23, 59, 60 ); tm.debugout(tmpfile);
    	tm.setgall( y-1, 5, 29, 23, 59, 60.01 ); tm.debugout(tmpfile);
    	tm.setgall( y-1, 5, 29, 23, 59, 59 ); tm.debugout(tmpfile);
    }

    fclose( tmpfile );
    t.check( system("diff leapseconds.tmp leapseconds.res") == 0 );

    return t.result();
}

/**************************************************************************************************************/
bool
SetGsecondTest()
{
    testt   t( "SetGsecondTest()" );
    ltimet  tm( 0 );
    gtimet  b1( 1968, 6, 29, 10, 20, 59 );
    gtimet  b2( 2002, 11, 30, 23, 59, 59 );
    
    for ( int y(1970); y < 2013; y++ )
    {
	gtimet	test1( y, 11, 30, 23, 59, 60 );
	gtimet	test2( y, 5, 29, 23, 59, 60 );
	bool	res( false );

	tm.setgall
	(
	    b1.gyear(),
	    b1.gmonth(),
	    b1.gmday(),
	    b1.ghour(),
	    b1.gminute(),
	    test1.durationfrom(b1) + b1.gsecond()
	);

	res = ( tm == test1 );

	tm.setgall
	(
	    b2.gyear(), 
	    b2.gmonth(), 
	    b2.gmday(), 
	    b2.ghour(), 
	    b2.gminute(), 
	    test1.durationfrom(b2) + b2.gsecond()
	);

	res &= ( tm == test1 );

	tm.setgall
	(
	    b1.gyear(), 
	    b1.gmonth(), 
	    b1.gmday(), 
	    b1.ghour(), 
	    b1.gminute(), 
	    test2.durationfrom(b1) + b1.gsecond()
	);

	res &= ( tm == test2 );

	tm.setgall
	(
	    b2.gyear(), 
	    b2.gmonth(), 
	    b2.gmday(), 
	    b2.ghour(), 
	    b2.gminute(), 
	    test2.durationfrom(b2) + b2.gsecond()
	);

	res &= ( tm == test2 );

	t.check( res );
    }

    return t.result();
}

/**************************************************************************************************************/
bool
SetGydayTest()
{
    testt   t( "SetGydayTest()" );
    ltimet  tm( 0 );

    //--- 1 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2000, 366-1, 0, 0, 0 );
    t.check( !tm.isdistorted() );
    
    //--- 2 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2001, 366-1, 0, 0, 0 );
    t.check( tm.isdistorted() );

    return t.result();
}

/**************************************************************************************************************/
bool
SetGmonthTest()
{
    testt      t( "SetGmonthTest()" );
    timetestt  tm;

    //--- 1 ----------------------------------------------------------------------------------------------------
    tm.setgall( 0, 0, 0, 0, 0, 0 );
    tm._setgyear( 2000 );  
    t.check( tm == gtimet(2000,0,0,0,0,0) );

    //--- 2 ----------------------------------------------------------------------------------------------------
    tm._setgmonth( 2-1 );  
    t.check( tm == gtimet(2000,1,0,0,0,0) );
    
    //--- 3 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2000, 2-1, 29-1, 0, 0, 0 );			// 2001-02-29
    t.check( !tm.isdistorted() );
    
    //--- 4 ----------------------------------------------------------------------------------------------------
    tm._setgyear( 2001 );					// 2001-02-28
    t.check( !tm.isdistorted(gtimet::YEARBIT) );
    
    //--- 5 ----------------------------------------------------------------------------------------------------
    tm._setgmonth( 12-1 );					// 2001-12-28
    t.check( !tm.isdistorted() );
    
    //--- 6 ----------------------------------------------------------------------------------------------------
    tm._setgmonth( 2-1 );					// 2001-02-28
    t.check( !tm.isdistorted() );

    //--- 7 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2000, 3-1, 13-1, 0, 0, 0 );
    tm._setgmonth( -1 );					// 1999-12-13
    t.check( tm == gtimet(1999,12-1,13-1,0,0,0) );

    //--- 8 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2001, 12-1, 31-1, 0, 0, 0 );
    t.check( !tm.isdistorted() );

    //--- 9 ----------------------------------------------------------------------------------------------------
    tm._setgmonth( 2-1 );					// 2001-02-28
    t.check( tm == gtimet(2001,2-1,28-1,0,0,0) );

    //--- 10 ---------------------------------------------------------------------------------------------------
    t.check( !tm.isdistorted(gtimet::MONTHBIT) );
 
    //--- 11 ---------------------------------------------------------------------------------------------------
    tm._setgmonth( -1 );					// 2000-12-28
    t.check( tm == gtimet(2000,12-1,28-1,0,0,0) );

    return t.result();
}

/**************************************************************************************************************/
bool
SetGyearTest()
{
    testt      t( "SetGyearTest()" );
    timetestt  tm;

    //--- 1 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2000, 2-1, 29-1, 0, 0, 0 );
    t.check( !tm.isdistorted() );
    
    //--- 2 ----------------------------------------------------------------------------------------------------
    tm._setgyear( 2001 );
    t.check( tm == gtimet(2001,2-1,28-1,0,0,0) );

    //--- 3 ----------------------------------------------------------------------------------------------------
    t.check( !tm.isdistorted(gtimet::YEARBIT) );

    //--- 4 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2001, 3-1, 12-1, 0, 0, 0 );
    t.check( !tm.isdistorted() );
    
    //--- 5 ----------------------------------------------------------------------------------------------------
    tm._setgyear( 2000 );
    t.check( tm == gtimet(2000,3-1,12-1,0,0,0) );
    
    //--- 6 ----------------------------------------------------------------------------------------------------
    t.check( !tm.isdistorted() );

    //--- 7 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2000, 3-1, 13-1, 0, 0, 0 );
    t.check( !tm.isdistorted() );
    
    //--- 8 ----------------------------------------------------------------------------------------------------
    tm._setgyear( 2001 );
    t.check( tm == gtimet(2001,3-1,13-1,0,0,0) );
    
    //--- 9 ----------------------------------------------------------------------------------------------------
    t.check( !tm.isdistorted() );

    //--- 10 ---------------------------------------------------------------------------------------------------
    tm.setgall( 2000, 12-1, 31-1, 0, 0, 0 );
    t.check( !tm.isdistorted() );
    
    //--- 11 ---------------------------------------------------------------------------------------------------
    tm._setgyear( 2001 );
    t.check( tm == gtimet(2001,12-1,31-1,0,0,0) );
    
    //--- 12 ---------------------------------------------------------------------------------------------------
    t.check( !tm.isdistorted() );

    //--- 13 ---------------------------------------------------------------------------------------------------
    tm.setgall( 2001, 12-1, 31-1, 0, 0, 0 );
    t.check( !tm.isdistorted() );
    
    //--- 14 ---------------------------------------------------------------------------------------------------
    tm._setgyear( 2000 );
    t.check( tm == gtimet( 2000,12-1,31-1,0,0,0) );
    
    //--- 15 ---------------------------------------------------------------------------------------------------
    t.check( !tm.isdistorted() );

    //--- 16 ---------------------------------------------------------------------------------------------------
    tm.setunixtime( 954090000 );				// 2000-03-27 00:00:00+07

    ltimet  c( tm );

    tm._setgyear( 2004 );

    t.check
    (
	tm.gyear() == 2004 &&
	tm.gmonth() == c.gmonth() &&
	tm.gmday() == c.gmday() &&
	tm.ghour() == c.ghour() &&
	tm.gminute() == c.gminute() &&
	tm.gsecond() == c.gsecond()
    );

    return t.result();
}

/**************************************************************************************************************/
bool
Y1970Test()
{
    testt   t( "Y1970Test()" );

    //--- 1 ----------------------------------------------------------------------------------------------------
    t.check( gtimet(1970,0,0,0,0,0) == gtimet(time_t(0)) );

    //--- 2 ----------------------------------------------------------------------------------------------------
    time_t  sec( 993273515L );
    gtimet  tm( sec );
    gtimet  ctm( tm.gyear(), tm.gmonth(), tm.gmday(), tm.ghour(), tm.gminute(), tm.gsecond() );

    t.check( ctm.unixtime() == sec );
    
    return t.result();
}

/**************************************************************************************************************/
bool
GydayLinearityTest()
{
    testt      t( "GydayLinearityTest()" );
    timetestt  tm;
    bool       res; 
    int        days;
    int        oldyday; 
    int        newyday;
    
    //--- 1 ----------------------------------------------------------------------------------------------------
    tm.setgall( 2000, 0, 0, 0, 0, 0 );
    oldyday = 0;
    res = true;

    for ( days=-1; days > -300000; days-- )
    {
	tm._setgdays( days );
	newyday = tm.gyday();
	
	if ( oldyday-newyday!=1 && (oldyday!=0 || (newyday!=364 && newyday!=365)) )
	{
	    res = false;
	    // cout << "gdays=" << days << " gyday=" << newyday << " oldyday=" << oldyday << " " << tm.str() << endl;

	    break;
	}
	
	oldyday = newyday;
    }
    
    t.check( res );

    //--- 2 ----------------------------------------------------------------------------------------------------
    oldyday = 0;
    res = true;

    for ( res=true,oldyday=0,days = 1; days < 300000; days++ )
    {
	tm._setgdays( days );
	newyday = tm.gyday();
	
	if ( newyday-oldyday!=1 && (newyday!=0 || (oldyday!=364 && oldyday!=365)) )
	{
	    res = false;
	    // cout << "gdays=" << days << " gyday=" << newyday << " oldyday=" << oldyday << " " << tm.str() << endl;

	    break;
	}
	
	oldyday = newyday;
    }
    
    t.check( res );

    return t.result();
}

/**************************************************************************************************************/
bool
GwdayTest()
{
    testt      t( "GwdayTest()" );
    timetestt  tm;

    //--- 1 ----------------------------------------------------------------------------------------------------
    tm.setgall( 1970, 0, 0, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::THU );

    //--- 2 ----------------------------------------------------------------------------------------------------
    tm.setgall( 1970, 1, 27, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::SAT );

    //--- 3 ----------------------------------------------------------------------------------------------------
    tm.setgall( 1970, 2, 0, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::SUN );

    //--- 4 ----------------------------------------------------------------------------------------------------
    tm.setgall( 1980, 0, 0, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::TUE );

    //--- 5 ----------------------------------------------------------------------------------------------------
    tm._setgmday( 1 );
    t.check( tm.gwday() == gtimet::WED );

    //--- 6 ----------------------------------------------------------------------------------------------------
    tm._setgmday( 2 );
    t.check( tm.gwday() == gtimet::THU );

    //--- 7 ----------------------------------------------------------------------------------------------------
    tm._setgmday( 3 );
    t.check( tm.gwday() == gtimet::FRI );

    //--- 8 ----------------------------------------------------------------------------------------------------
    tm._setgmday( 4 );
    t.check( tm.gwday() == gtimet::SAT );

    //--- 9 ----------------------------------------------------------------------------------------------------
    tm.setgall( 1980, 1, 28, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::FRI );

    //--- 10 ---------------------------------------------------------------------------------------------------
    tm.setgall( 1980, 2, 0, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::SAT );

    //--- 11 ---------------------------------------------------------------------------------------------------
    tm.setgall( 1900, 0, 0, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::MON );

    //--- 12 ---------------------------------------------------------------------------------------------------
    tm.setgall( 1600, 0, 0, 0, 0, 0 );
    t.check( tm.gwday() == gtimet::SAT );

    return t.result();
}

/**************************************************************************************************************/
bool
GyearTest()
{
    testt      t( "GyearTest()" );
    timetestt  tm;
    bool       res( true );
    long       days( -150845 );	// 1587-01-01 00:00:00+0000
    
    for ( int i=0; i<1240; i++ )
    {
	days += 365;
	
	if ( (i+1587)%4 == 0 && ((i+1587)%100!=0 || (i+1587)%400==0) )
	{
	    days++;
	}
	
	tm.setgall( 0, 0, 0, 0, 0, 0 );

	gtimet	tm1( tm );
	
	tm._setgdays( days );
	tm1 = tm;
	tm1 -= 0.001L;
	
	if 
	(
	    tm.gmonth() != 1-1 ||
	    tm.gmday() != 1-1 ||
	    tm.ghour() != 0 ||
	    tm.gminute() != 0 ||
	    tm.gsecond() != 0.0L ||
	    tm1.gmonth() != 12-1 ||
	    tm1.gmday() != 31-1 ||
	    tm1.ghour() != 23 ||
	    tm1.gminute() != 59 ||
	    tm1.gsecond() < 58
	)
	{ 
	    res = false;
/*
	    cout <<
		tm.str() << " " << tm1.str() << " " << i+1587 << " " << i << " " <<
		tm.gdays() << " " << tm.gdays()-days << " " << tm1.gyday() << endl;
*/
	}
    }

    t.check( res );

    return t.result();
}

/**************************************************************************************************************/
bool
JulianDateTest()
{
    testt   t( "JulianDateTest()" );
    ltimet  tm( "2000-01-01 00:00:00Z" );

    tm.setjuliandate( 2451545.0L );

    //--- 1 ----------------------------------------------------------------------------------------------------
    t.check( tm.gyear() == 2000.0L );

    //--- 2 ----------------------------------------------------------------------------------------------------
    t.check( tm.gmonth() == 0 );

    //--- 3 ----------------------------------------------------------------------------------------------------
    t.check( tm.gmday() == 0 );

    //--- 4 ----------------------------------------------------------------------------------------------------
    t.check( tm.ghour() == 11 );

    //--- 5 ----------------------------------------------------------------------------------------------------
    t.check( tm.gminute() == 58 );

    //--- 6 ----------------------------------------------------------------------------------------------------
    // tm.gsecond() == 55.816L;
    t.check( floor(tm.gsecond()*100000000000.0L+0.5L) == 5581600000000.0L );
    
    tm.setjuliandate( 0.0L );
    
    //--- 7 ----------------------------------------------------------------------------------------------------
    t.check( tm.gyear() == -4713.0L );

    //--- 8 ----------------------------------------------------------------------------------------------------
    t.check( tm.gmonth() == 10 );

    //--- 9 ----------------------------------------------------------------------------------------------------
    t.check( tm.gmday() == 23 );

    //--- 10 ---------------------------------------------------------------------------------------------------
    t.check( tm.ghour() == 11 );

    //--- 11 ---------------------------------------------------------------------------------------------------
    t.check( tm.gminute() == 59 );

    //--- 12 ---------------------------------------------------------------------------------------------------
    // gsecond() == 17.816L;
    t.check( floor(tm.gsecond()*10000.0L+0.5L) == 178160.0L );

    return t.result();
}

/**************************************************************************************************************/
bool
LocaltimeTest( string Tz )
{
    ltimet  tm( 0 );

    tm.settz( Tz );

    testt  t( string("LocaltimeTest( ") + tm.tz() + " )" );

    //--- 1 --- tzname[0] == ALMT ------------------------------------------------------------------------------
    t.check( string(tzname[0]) == "ALMT" );

    //--- 2 --- tzname[1] == ALMST -----------------------------------------------------------------------------
    t.check( string(tzname[1]) == "ALMST" );

    //--- 3 --- 2001-03-24 19:59:59 UTC == 2001-03-25 01:59:59 ALMT --------------------------------------------
    gtimet  galmt3( 2001, 3-1, 24-1, 19, 59, 59 );
    ltimet  almt3( galmt3 );

    t.check
    (
	almt3.lyear() == 2001 &&
	almt3.lmonth() == 3-1 &&
	almt3.lmday() == 25-1 &&
	almt3.lhour() == 01 &&
	almt3.lminute() == 59 &&
	almt3.lsecond() == 59 &&
	almt3.ltzname() == "ALMT"
    );

    //--- 4 --- 2001-03-24 20:00:00 UTC == 2001-03-25 03:00:00 ALMST -------------------------------------------
    gtimet  galmt4( 2001, 3-1, 24-1, 20, 0, 0 );
    ltimet  almt4( galmt4 );
	
    t.check
    (
	almt4.lyear() == 2001 &&
	almt4.lmonth() == 3-1 &&
	almt4.lmday() == 25-1 &&
	almt4.lhour() == 3 &&
	almt4.lminute() == 0 &&
	almt4.lsecond() == 0 &&
	almt4.ltzname() == "ALMST"
    );

    //--- 5 --- 2001-10-27 19:59:59 UTC == 2001-10-28 02:59:59 ALMST -------------------------------------------
    gtimet  galmst5( 2001, 10-1, 27-1, 19, 59, 59 );
    ltimet  almst5( galmst5 );
	
    t.check
    (
	almst5.lyear() == 2001 &&
	almst5.lmonth() == 10-1 &&
	almst5.lmday() == 28-1 &&
	almst5.lhour() == 2 &&
	almst5.lminute() == 59 &&
	almst5.lsecond() == 59 &&
	almst5.ltzname() == "ALMST"
    );

    //--- 6 --- 2001-10-27 20:00:00 UTC == 2001-10-28 02:00:00 ALMT --------------------------------------------
    gtimet  galmst6( 2001, 10-1, 27-1, 20, 0, 0 );
    ltimet  almst6( galmst6 );
	
    t.check
    (
	almst6.lyear() == 2001 &&
	almst6.lmonth() == 10-1 &&
	almst6.lmday() == 28-1 &&
	almst6.lhour() == 2 &&
	almst6.lminute() == 0 &&
	almst6.lsecond() == 0 &&
	almst6.ltzname() == "ALMT"
    );

    //--- 7 --- (2001-03-25 02:00:00) 2001-03-25 03:00:00 ALMST == 2001-03-24 20:00:00 UTC distorted=t ---------
    tm.setlall( 2001, 3-1, 25-1, 2, 0, 0 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 3-1 &&
	tm.lmday() == 25-1 &&
	tm.lhour() == 3 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 3-1 &&
	tm.gmday() == 24-1 &&
	tm.ghour() == 20 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted()
    );

    //--- 8 --- (2001-03-25 02:59:59) 2001-03-25 03:00:00 ALMST == 2001-03-24 20:00:00 UTC distorted=t ---------
    tm.setlall( 2001, 3-1, 25-1, 2, 59, 59 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 3-1 &&
	tm.lmday() == 25-1 &&
	tm.lhour() == 3 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 3-1 &&
	tm.gmday() == 24-1 &&
	tm.ghour() == 20 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted()
    );

    //--- 9 --- (2001-03-25 02:59:60) 2001-03-25 03:00:00 ALMST == 2001-03-24 20:00:00 UTC distorted=t ---------
    tm.setlall( 2001, 3-1, 25-1, 2, 59, 60 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 3-1 &&
	tm.lmday() == 25-1 &&
	tm.lhour() == 3 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 3-1 &&
	tm.gmday() == 24-1 &&
	tm.ghour() == 20 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted()
    );

    //--- 10 --- (2001-03-25 03:00:00) 2001-03-25 03:00:00 ALMST == 2001-03-24 20:00:00 UTC distorted=f --------
    tm.setlall( 2001, 3-1, 25-1, 3, 0, 0 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 3-1 &&
	tm.lmday() == 25-1 &&
	tm.lhour() == 3 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 3-1 &&
	tm.gmday() == 24-1 &&
	tm.ghour() == 20 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted() == false
    );

    //--- 11 --- (2001-03-25 03:00:01) 2001-03-25 03:00:01 ALMST == 2001-03-24 20:00:01 UTC distorted=f --------
    tm.setlall( 2001, 3-1, 25-1, 3, 0, 1 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 3-1 &&
	tm.lmday() == 25-1 &&
	tm.lhour() == 3 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 1 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 3-1 &&
	tm.gmday() == 24-1 &&
	tm.ghour() == 20 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 1 &&
	tm.isdistorted() == false
    );

    //--- 12 --- (2001-10-28 01:59:59) 2001-10-28 01:59:59 ALMST == 2001-10-27 18:59:59 UTC distorted=f --------
    tm.setlall( 2001, 10-1, 28-1, 1, 59, 59 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 10-1 &&
	tm.lmday() == 28-1 &&
	tm.lhour() == 1 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 59 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 27-1 &&
	tm.ghour() == 18 &&
	tm.gminute() == 59 &&
	tm.gsecond() == 59 &&
	tm.isdistorted() == false
    );

    //--- 13 --- (2001-10-28 02:00:00) 2001-10-28 02:00:00 ALMST == 2001-10-27 19:00:00 UTC distorted=f --------
    tm.setlall( 2001, 10-1, 28-1, 2, 0, 0 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 10-1 &&
	tm.lmday() == 28-1 &&
	tm.lhour() == 2 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 27-1 &&
	tm.ghour() == 19 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted() == false
    );

    //--- 14 --- (2001-10-28 02:00:01) 2001-10-28 02:00:01 ALMST == 2001-10-27 19:00:01 UTC distorted=f --------
    tm.setlall( 2001, 10-1, 28-1, 2, 0, 1 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 10-1 &&
	tm.lmday() == 28-1 &&
	tm.lhour() == 2 &&
	tm.lminute() == 0 &&
 	tm.lsecond() == 1 &&
	tm.ltzname() == "ALMST" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 27-1 &&
	tm.ghour() == 19 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 1 &&
	tm.isdistorted() == false
    );

    //--- 15 -- (2001-10-28 02:59:59) 2001-10-28 02:59:59 ALM[S]T == 2001-10-27 (20|19):59:59 UTC distorted=f --
    //
    // Some ambiguity take place. Clock hands pass the 2001-10-28 02:59:59 value twice. First time as ALMST,
    // second time as ALMT. Cause the input data has no time zone information both values are right.
    //
    tm.setlall( 2001, 10-1, 28-1, 02, 59, 59 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 10-1 &&
	tm.lmday() == 28-1 &&
	tm.lhour() == 2 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 59 &&
	(tm.ltzname() == "ALMST" || tm.ltzname() == "ALMT") &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 27-1 &&
	(tm.ghour() == 19 || tm.ghour() == 20) &&
	tm.gminute() == 59 &&
	tm.gsecond() == 59 &&
	tm.isdistorted() == false
    );

    //--- 16 --- (2001-10-28 03:00:00) 2001-10-28 03:00:00 ALMT == 2001-10-27 21:00:00 UTC distorted=f ---------
    tm.setlall( 2001, 10-1, 28-1, 3, 0, 0 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 10-1 &&
	tm.lmday() == 28-1 &&
	tm.lhour() == 3 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMT" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 27-1 &&
	tm.ghour() == 21 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted() == false
    );

    //--- 17 --- (2001-10-28 03:00:01) 2001-10-28 03:00:01 ALMT == 2001-10-27 21:00:01 UTC distorted=f ---------
    tm.setlall( 2001, 10-1, 28-1, 3, 0, 1 );

    t.check
    (
	tm.lyear() == 2001 &&
	tm.lmonth() == 10-1 &&
	tm.lmday() == 28-1 &&
	tm.lhour() == 3 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 1 &&
	tm.ltzname() == "ALMT" &&
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 27-1 &&
	tm.ghour() == 21 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 1 &&
	tm.isdistorted() == false
     );

    //--- 18 --- (2001-01-01 00:59:59) 2001-01-01 00:59:59 ALMT == 2000-12-31 18:59:59 UTC distorted=f ---------
    tm.setlall( 2001, 1-1, 1-1, 0, 59, 59 );

    t.check
    (
 	tm.lyear() == 2001 &&
	tm.lmonth() == 1-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 0 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 59 &&
	tm.ltzname() == "ALMT" &&
	tm.gyear() == 2000 &&
	tm.gmonth() == 12-1 &&
	tm.gmday() == 31-1 &&
	tm.ghour() == 18 &&
	tm.gminute() == 59 &&
	tm.gsecond() == 59 &&
	tm.isdistorted() == false
    );

    //--- 19 --- (2004-12-32 00:00:00) 2005-01-01 00:00:00 ALMT == 2004-12-31 18:00:00 UTC distorted=t ---------
    tm.setlall( 2004, 12-1, 32-1, 0, 0, 0 );

    t.check
    (
 	tm.lyear() == 2005 &&
	tm.lmonth() == 1-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 0 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMT" &&
	tm.gyear() == 2004 &&
	tm.gmonth() == 12-1 &&
	tm.gmday() == 31-1 &&
	tm.ghour() == 18 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted() == true
    );

    //--- 20 --- (2005-01-00 00:00:00) 2004-12-31 00:00:00 ALMT == 2004-12-30 18:00:00 UTC distorted=t ---------
    tm.setlall( 2005, 1-1, 0-1, 0, 0, 0 );

    t.check
    (
 	tm.lyear() == 2004 &&
	tm.lmonth() == 12-1 &&
	tm.lmday() == 31-1 &&
	tm.lhour() == 0 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0 &&
	tm.ltzname() == "ALMT" &&
	tm.gyear() == 2004 &&
	tm.gmonth() == 12-1 &&
	tm.gmday() == 30-1 &&
	tm.ghour() == 18 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0 &&
	tm.isdistorted() == true
    );

    //--- 21 --- (0201-01-01 00:59:59) 0201-01-01 00:59:59 ALMT == 0200-12-31 18:59:59 UTC distorted=f ---------
    tm.setlall( 201, 1-1, 1-1, 0, 59, 59 );

    t.check
    (
	tm.lyear() == 201 &&
	tm.lmonth() == 1-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 0 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 59 &&
	tm.ltzname() == "ALMT" &&
	tm.gyear() == 200 &&
	tm.gmonth() == 12-1 &&
	tm.gmday() == 31-1 &&
	tm.ghour() == 18 &&
	tm.gminute() == 59 &&
	tm.gsecond() == 59 &&
	tm.isdistorted() == false
    );

    return t.result();
}

/**************************************************************************************************************/
bool
String2TimeTest( string Tz )
{
    ltimet  tm( 0 );

    tm.settz( Tz );

    testt   t( string("String2TimeTest( ") + tm.tz() + " )" );
    string  errdesc;
    ltimet  today( time(NULL) );

    //--- 1 ----------------------------------------------------------------------------------------------------
    tm.setall( "2001-10-15 T23:45:16.5+0600", &errdesc );

    t.check
    (
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 15-1 &&
	tm.ghour() == 17 &&
	tm.gminute() == 45 &&
	tm.gsecond() == 16.5 &&
	tm.isdistorted() == false
    );

    //--- 2 ----------------------------------------------------------------------------------------------------
    tm.setall( "2001-10-15 T23:45:16.5+06:00", &errdesc );

    t.check
    (
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 15-1 &&
	tm.ghour() == 17 &&
	tm.gminute() == 45 &&
	tm.gsecond() == 16.5 &&
	tm.isdistorted() == false
    );
    
    //--- 3 ----------------------------------------------------------------------------------------------------
    tm.setall( "2001-10-15 T23:45:16.5 +0600", &errdesc );

    t.check
    (
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 15-1 &&
	tm.ghour() == 17 &&
	tm.gminute() == 45 &&
	tm.gsecond() == 16.5 &&
	tm.isdistorted() == false
    );
    
    //--- 4 ----------------------------------------------------------------------------------------------------
    tm.setall( "-2001-10-15 T23:45:16.5+0630", &errdesc );

    t.check
    (
	tm.gyear() == -2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 15-1 &&
	tm.ghour() == 17 &&
	tm.gminute() == 15 &&
	tm.gsecond() == 16.5 &&
	tm.isdistorted() == false
    );
    
    //--- 5 ----------------------------------------------------------------------------------------------------
    tm.setall( "-2001-10-15 23:45:16.5", &errdesc );

    t.check
    (
	tm.gyear() == -2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 15-1 &&
	tm.ghour() == 17 &&
	tm.gminute() == 45 &&
	tm.gsecond() == 16.5 &&
	tm.isdistorted() == false
    );
    
    //--- 6 ----------------------------------------------------------------------------------------------------
    tm.setall( "2001-10-15 23:45:16.5", &errdesc );

    t.check
    (
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 15-1 &&
	tm.ghour() == 16 &&
	tm.gminute() == 45 &&
	tm.gsecond() == 16.5 &&
	tm.isdistorted() == false
    );
    
    //--- 7 ----------------------------------------------------------------------------------------------------
    tm.setall( "2001-10-15 T23:45:16.5", &errdesc );

    t.check
    (
	tm.gyear() == 2001 &&
	tm.gmonth() == 10-1 &&
	tm.gmday() == 15-1 &&
	tm.ghour() == 16 &&
	tm.gminute() == 45 &&
	tm.gsecond() == 16.5 &&
	tm.lyear() == 2001 &&
	tm.lmonth() == 10-1 &&
	tm.lmday() == 15-1 &&
	tm.lhour() == 23 &&
	tm.lminute() == 45 &&
	tm.lsecond() == 16.5 &&
	tm.isdistorted() == false
    );
    
    //--- 8 ----------------------------------------------------------------------------------------------------
    tm.setall( " -infinity ", &errdesc );

    t.check
    (
	tm.isabsolutebegin() &&
	tm.isdistorted() == false &&
	tm.gtimet::str() == "-infinity" &&
	tm.ltimet::str() == "-infinity"
    );

    //--- 9 ----------------------------------------------------------------------------------------------------
    tm.setall( "absolute_begin ", &errdesc );

    t.check
    (
	tm.isabsolutebegin() &&
	tm.isdistorted() == false &&
	tm.gtimet::str() == "-infinity" &&
	tm.ltimet::str() == "-infinity"
    );

    //--- 10 ---------------------------------------------------------------------------------------------------
    tm.setall( "absolutebegin", &errdesc );

    t.check
    (
	tm.isabsolutebegin() &&
	tm.isdistorted() == false &&
	tm.gtimet::str() == "-infinity" &&
	tm.ltimet::str() == "-infinity"
    );

    //--- 11 ---------------------------------------------------------------------------------------------------
    tm.setall( "infinity", &errdesc );

    t.check
    (
	tm.isabsoluteend() &&
	tm.isdistorted() == false &&
	tm.gtimet::str() == "infinity" &&
	tm.ltimet::str() == "infinity"
    );

    //--- 12 ---------------------------------------------------------------------------------------------------
    tm.setall( "absolute_end  ", &errdesc );

    t.check
    (
	tm.isabsoluteend() &&
	tm.isdistorted() == false &&
	tm.gtimet::str() == "infinity" &&
	tm.ltimet::str() == "infinity"
    );

    //--- 13 ---------------------------------------------------------------------------------------------------
    tm.setall( "\t    absoluteend", &errdesc );

    t.check
    (
	tm.isabsoluteend() &&
	tm.isdistorted() == false &&
	tm.gtimet::str() == "infinity" &&
	tm.ltimet::str() == "infinity"
    );

    //--- 14 ---------------------------------------------------------------------------------------------------
    tm.setall( "04:59", &errdesc );

    t.check
    (
	tm.gminute() == 59 &&
	tm.gsecond() == 0.0 &&
	tm.lyear() == today.lyear() &&
	tm.lmonth() == today.lmonth() &&
	tm.lmday() == today.lmday() &&
	tm.lhour() == 4 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );
    
    //--- 15 ---------------------------------------------------------------------------------------------------
    tm.setall( "04:59Z", &errdesc );

    t.check
    (
	tm.gyear() == today.gyear() &&
	tm.gmonth() == today.gmonth() &&
	tm.gmday() == today.gmday() &&
	tm.ghour() == 4 &&
	tm.gminute() == 59 &&
	tm.gsecond() == 0.0 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );
    
    //--- 16 ---------------------------------------------------------------------------------------------------
    tm.setall( "07:59+06", &errdesc );

    t.check
    (
	tm.gyear() == today.gyear() &&
	tm.gmonth() == today.gmonth() &&
	tm.gmday() == today.gmday() &&
	tm.ghour() == 1 &&
	tm.gminute() == 59 &&
	tm.gsecond() == 0.0 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );
 
    //--- 17 ---------------------------------------------------------------------------------------------------
    tm.setall( "2003-02-01", &errdesc );

    t.check
    (
	tm.gyear() == 2003 &&
	tm.gmonth() == 1-1 &&
	tm.gmday() == 31-1 &&
	tm.ghour() == 18 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0.0 &&
	tm.lyear() == 2003 &&
	tm.lmonth() == 2-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 0 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );

    //--- 18 ---------------------------------------------------------------------------------------------------
    tm.setall( " 2003-02-01TZ ", &errdesc );

    t.check
    (
	tm.gyear() == 2003 &&
	tm.gmonth() == 2-1 &&
	tm.gmday() == 1-1 &&
	tm.ghour() == 0 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0.0 &&
	tm.lyear() == 2003 &&
	tm.lmonth() == 2-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 6 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );
    
    //--- 19 ---------------------------------------------------------------------------------------------------
    tm.setall( "2003-02-01 Z", &errdesc );

    t.check
    (
	tm.gyear() == 2003 &&
	tm.gmonth() == 2-1 &&
	tm.gmday() == 1-1 &&
	tm.ghour() == 0 &&
	tm.gminute() == 0 &&
	tm.gsecond() == 0.0 &&
	tm.lyear() == 2003 &&
	tm.lmonth() == 2-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 6 &&
	tm.lminute() == 0 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );

    //--- 20 ---------------------------------------------------------------------------------------------------
    tm.setall( "1997-07-01 04:59", &errdesc );

    t.check
    (
	tm.gminute() == 59 &&
	tm.gsecond() == 0.0 &&
	tm.lyear() == 1997 &&
	tm.lmonth() == 7-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 4 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );

    //--- 21 ---------------------------------------------------------------------------------------------------
    tm.setall( "2012-07-01 04:59", &errdesc );

    t.check
    (
	tm.gminute() == 59 &&
	tm.gsecond() == 0.0 &&
	tm.lyear() == 2012 &&
	tm.lmonth() == 7-1 &&
	tm.lmday() == 1-1 &&
	tm.lhour() == 4 &&
	tm.lminute() == 59 &&
	tm.lsecond() == 0.0 &&
	tm.isdistorted() == false
    );
    
    return t.result();
}

/**************************************************************************************************************/
bool
ComparisonTest()
{
    testt   t( "ComparisonTest()" );
    ltimet  tm( 0 );
    ltimet  before( "2005-12-31 23:59:59 Z" );
    ltimet  appoint( "2005-12-31 23:59:60 Z" );
    ltimet  past( "2006-01-01 00:00:00 Z" );

    tm = appoint;

    //--- 1 ----------------------------------------------------------------------------------------------------
    t.check( before < appoint );

    //--- 2 ----------------------------------------------------------------------------------------------------
    t.check( appoint < past );

    //--- 3 ----------------------------------------------------------------------------------------------------
    t.check( before < past );

    //--- 4 ----------------------------------------------------------------------------------------------------
    t.check( before <= appoint );

    //--- 5 ----------------------------------------------------------------------------------------------------
    t.check( appoint <= past );

    //--- 6 ----------------------------------------------------------------------------------------------------
    t.check( before <= past );

    //--- 7 ----------------------------------------------------------------------------------------------------
    t.check( tm <= appoint );

    //--- 8 ----------------------------------------------------------------------------------------------------
    t.check( tm == appoint );

    //--- 9 ----------------------------------------------------------------------------------------------------
    t.check( tm >= appoint );

    //--- 10 ---------------------------------------------------------------------------------------------------
    t.check( tm != before );

    //--- 11 ---------------------------------------------------------------------------------------------------
    t.check( tm != past );

    //--- 12 ---------------------------------------------------------------------------------------------------
    t.check( before != past );

    //--- 13 ---------------------------------------------------------------------------------------------------
    t.check( past > appoint );

    //--- 14 ---------------------------------------------------------------------------------------------------
    t.check( appoint > before );

    //--- 15 ---------------------------------------------------------------------------------------------------
    t.check( past > before );

    //--- 16 ---------------------------------------------------------------------------------------------------
    t.check( past >= appoint );

    //--- 17 ---------------------------------------------------------------------------------------------------
    t.check( appoint >= before );

    //--- 18 ---------------------------------------------------------------------------------------------------
    t.check( past >= before );

    return t.result();
}

/**************************************************************************************************************/
void  precisions()
{
    cout << "precisions():" << endl;

    long double  sec( 0.123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789L );
    gtimet	 test1( 2000, 0, 0, 0, 0, sec );
    gtimet	 test2( 1999, 10, 0, 0, 0, sec );
    gtimet	 test3( 1999, 0, 0, 0, 0, sec );
    gtimet	 test4( 1998, 0, 0, 0, 0, sec );
    gtimet	 test5( 1995, 0, 0, 0, 0, sec );
    gtimet	 test6( 1970, 0, 0, 0, 0, sec );
    gtimet	 test7( 0, 0, 0, 0, 0, sec );
    gtimet	 test8( 20000, 0, 0, 0, 0, sec );
    gtimet	 test9( 1000000, 0, 0, 0, 0, sec );
    gtimet	 test10( 100000000, 0, 0, 0, 0, sec );
    gtimet	 test11( -20000, 0, 0, 0, 0, sec );
    
    cout << "\t2000-01-01\t" << floor(-log10(fabs(sec-test1.gsecond())))-1 << endl;
    cout << "\t1999-11-01\t" << floor(-log10(fabs(sec-test2.gsecond())))-1 << endl;
    cout << "\t1999-01-01\t" << floor(-log10(fabs(sec-test3.gsecond())))-1 << endl;
    cout << "\t1998-01-01\t" << floor(-log10(fabs(sec-test4.gsecond())))-1 << endl;
    cout << "\t1995-01-01\t" << floor(-log10(fabs(sec-test5.gsecond())))-1 << endl;
    cout << "\t1970-01-01\t" << floor(-log10(fabs(sec-test6.gsecond())))-1 << endl;
    cout << "\t0000-01-01\t" << floor(-log10(fabs(sec-test7.gsecond())))-1 << endl;
    cout << "\t20000-01-01\t" << floor(-log10(fabs(sec-test8.gsecond())))-1 << endl;
    cout << "\t1000000-01-01\t" << floor(-log10(fabs(sec-test9.gsecond())))-1 << endl;
    cout << "\t100000000-01-01\t" << floor(-log10(fabs(sec-test10.gsecond())))-1 << endl;
    cout << "\t-20000-01-01\t" << floor(-log10(fabs(sec-test11.gsecond())))-1 << endl;
}    

/**************************************************************************************************************/
int
main()
{
    TimezoneTest();
    TZLeapSecondTest( ":UTC" );
    TZLeapSecondTest( ":Almaty" );
    TZLeapSecondTest( ":Almaty_r" );
    LeapSecondTest();
    SetGsecondTest();
    SetGydayTest();
    SetGmonthTest();
    SetGyearTest();
    Y1970Test();
    GydayLinearityTest();
    GwdayTest();
    GyearTest();
    JulianDateTest();
    LocaltimeTest( ":Almaty" );
    LocaltimeTest( ":Almaty_r" );
    String2TimeTest( ":Almaty" );
    String2TimeTest( ":Almaty_r" );
    ComparisonTest();

    //precisions();

    return 0;
}
